"""
Interface to OpenSSL BIO library
"""
from ctypescrypto import libcrypto
from ctypes import c_char_p, c_void_p, c_int, string_at, c_long
from ctypes import POINTER, byref, create_string_buffer
class Membio(object):
    """
    Provides interface to OpenSSL memory bios
    use str() or unicode() to get contents of writable bio
    use bio member to pass to libcrypto function
    """
    def __init__(self, data=None):
        """
        If data is specified, creates read-only BIO. If data is
        None, creates writable BIO, contents of which can be retrieved
        by str() or unicode()
        """
        if data is None:
            method = libcrypto.BIO_s_mem()
            self.bio = libcrypto.BIO_new(method)
        else:
            self.bio = libcrypto.BIO_new_mem_buf(c_char_p(data), len(data))

    def __del__(self):
        """
        Cleans up memory used by bio
        """
        libcrypto.BIO_free(self.bio)
        del self.bio

    def __str__(self):
        """
        Returns current contents of buffer as byte string
        """
        string_ptr = c_char_p(None)
        string_len = libcrypto.BIO_ctrl(self.bio, 3, 0, byref(string_ptr))
        return string_at(string_ptr, string_len)

    def __unicode__(self):
        """
        Attempts to interpret current contents of buffer as UTF-8 string
        and convert it to unicode
        """
        return str(self).decode("utf-8")

    def read(self, length=None):
        """
        Reads data from readble BIO. For test purposes.
        @param length - if specifed, limits amount of data read.
        If not BIO is read until end of buffer
        """
        if not length is None:
            if not isinstance(length, (int, long)):
                raise TypeError("length to read should be number")
            buf = create_string_buffer(length)
            readbytes = libcrypto.BIO_read(self.bio, buf, length)
            if readbytes == -2:
                raise NotImplementedError("Function is not supported by" +
                                          "this BIO")
            if readbytes == -1:
                raise IOError
            if readbytes == 0:
                return ""
            return buf.raw[:readbytes]
        else:
            buf = create_string_buffer(1024)
            out = ""
            readbytes = 1
            while readbytes > 0:
                readbytes = libcrypto.BIO_read(self.bio, buf, 1024)
                if readbytes == -2:
                    raise NotImplementedError("Function is not supported by " +
                                              "this BIO")
                if readbytes == -1:
                    raise IOError
                if readbytes > 0:
                    out += buf.raw[:readbytes]
            return out

    def write(self, data):
        """
        Writes data to writable bio. For test purposes
        """
        if isinstance(data, unicode):
            data = data.encode("utf-8")
        else:
            data = str(data)

        written = libcrypto.BIO_write(self.bio, data, len(data))
        if written == -2:
            raise NotImplementedError("Function not supported by this BIO")
        if written < len(data):
            raise IOError("Not all data were successfully written")

    def reset(self):
        """
        Resets the read-only bio to start and discards all data from
        writable bio
        """
        libcrypto.BIO_ctrl(self.bio, 1, 0, None)

__all__ = ['Membio']
libcrypto.BIO_s_mem.restype = c_void_p
libcrypto.BIO_new.restype = c_void_p
libcrypto.BIO_new.argtypes = (c_void_p, )
libcrypto.BIO_ctrl.restype = c_long
libcrypto.BIO_ctrl.argtypes = (c_void_p, c_int, c_long, POINTER(c_char_p))
libcrypto.BIO_read.argtypes = (c_void_p, c_char_p, c_int)
libcrypto.BIO_write.argtypes = (c_void_p, c_char_p, c_int)
libcrypto.BIO_free.argtypes = (c_void_p, )
libcrypto.BIO_new_mem_buf.restype = c_void_p
libcrypto.BIO_new_mem_buf.argtypes = (c_char_p, c_int)



"""
Implements interface to openssl X509 and X509Store structures,
I.e allows to load, analyze and verify certificates.

X509Store objects are also used to verify other signed documets,
such as CMS, OCSP and timestamps.
"""



from ctypes import c_void_p, c_long, c_int, POINTER, c_char_p, Structure, cast
from ctypescrypto.bio import Membio
from ctypescrypto.pkey import PKey
from ctypescrypto.oid import Oid
from ctypescrypto.exception import LibCryptoError
from ctypescrypto import libcrypto
from datetime import datetime

try:
    from pytz import utc
except ImportError:
    from datetime import timedelta, tzinfo
    ZERO = timedelta(0)
    class UTC(tzinfo):
        """tzinfo object for UTC.
            If no pytz is available, we would use it.
        """
        def utcoffset(self, dt):
            return ZERO

        def tzname(self, dt):
            return "UTC"

        def dst(self, dt):
            return ZERO

    utc = UTC()

__all__ = ['X509', 'X509Error', 'X509Name', 'X509Store', 'StackOfX509']

if hasattr(libcrypto,"X509_get_version"):
    
# If it is OpenSSL 1.1 or above, use accessor functions
    _X509_get_version = libcrypto.X509_get_version
    _X509_get_version.restype = c_long
    _X509_get_version.argtypes = (c_void_p,)

    _X509_get_notBefore=libcrypto.X509_get_notBefore
    _X509_get_notBefore.restype = c_void_p
    _X509_get_notBefore.argtypes = (c_void_p,)

    _X509_get_notAfter=libcrypto.X509_get_notAfter
    _X509_get_notAfter.restype = c_void_p
    _X509_get_notAfter.argtypes = (c_void_p,)
else:
    # Otherwise declare X509 structure internals and define deep poke
    # functions
    class _validity(Structure):
        """ ctypes representation of X509_VAL structure
            needed to access certificate validity period, because openssl
            doesn't provide fuctions for it - only macros
        """
        _fields_ = [('notBefore', c_void_p), ('notAfter', c_void_p)]

    class _cinf(Structure):
        """ ctypes representtion of X509_CINF structure
            neede to access certificate data, which are accessable only
            via macros
        """
        _fields_ = [('version', c_void_p),
                    ('serialNumber', c_void_p),
                    ('sign_alg', c_void_p),
                    ('issuer', c_void_p),
                    ('validity', POINTER(_validity)),
                    ('subject', c_void_p),
                    ('pubkey', c_void_p),
                    ('issuerUID', c_void_p),
                    ('subjectUID', c_void_p),
                    ('extensions', c_void_p),
                ]

    class _x509(Structure):
        """
        ctypes represntation of X509 structure needed
        to access certificate data which are accesable only via
        macros, not functions
        """
        _fields_ = [('cert_info', POINTER(_cinf)),
                    ('sig_alg', c_void_p),
                    ('signature', c_void_p),
                    # There are a lot of parsed extension fields there
                ]
    _px509 = POINTER(_x509)
    def _X509_get_version(ptr):
        asn1int = cast(ptr, _px509)[0].cert_info[0].version  
        return libcrypto.ASN1_INTEGER_get(asn1int)

    def _X509_get_notBefore(ptr):   
        # (x)->cert_info->validity->notBefore
        return cast(ptr, _px509)[0].cert_info[0].validity[0].notBefore
    def _X509_get_notAfter(ptr):
        return cast(ptr, _px509)[0].cert_info[0].validity[0].notAfter

class X509Error(LibCryptoError):
    """
    Exception, generated when some openssl function fail
    during X509 operation
    """
    pass


class X509Name(object):
    """
    Class which represents X.509 distinguished name - typically
    a certificate subject name or an issuer name.

    Now used only to represent information, extracted from the
    certificate. Potentially can be also used to build DN when creating
    certificate signing request
    """
    # XN_FLAG_SEP_COMMA_PLUS & ASN1_STRFLG_UTF8_CONVERT
    PRINT_FLAG = 0x10010
    ESC_MSB = 4
    def __init__(self, ptr=None, copy=False):
        """
        Creates a X509Name object
        @param ptr - pointer to X509_NAME C structure (as returned by some
                     OpenSSL functions
        @param copy - indicates that this structure have to be freed upon
                      object destruction
        """
        if ptr is not None:
            self.ptr = ptr
            self.need_free = copy
            self.writable = False
        else:
            self.ptr = libcrypto.X509_NAME_new()
            self.need_free = True
            self.writable = True

    def __del__(self):
        """
        Frees if neccessary
        """
        if self.need_free:
            libcrypto.X509_NAME_free(self.ptr)
    def __str__(self):
        """
        Produces an ascii representation of the name, escaping all
        symbols > 0x80.  Probably it is not what you want, unless
        your native language is English
        """
        bio = Membio()
        libcrypto.X509_NAME_print_ex(bio.bio, self.ptr, 0,
                                     self.PRINT_FLAG | self.ESC_MSB)
        return str(bio)

    def __unicode__(self):
        """
        Produces unicode representation of the name.
        """
        bio = Membio()
        libcrypto.X509_NAME_print_ex(bio.bio, self.ptr, 0, self.PRINT_FLAG)
        return unicode(bio)
    def __len__(self):
        """
        return number of components in the name
        """
        return libcrypto.X509_NAME_entry_count(self.ptr)
    def __cmp__(self, other):
        """
        Compares X509 names
        """
        return libcrypto.X509_NAME_cmp(self.ptr, other.ptr)
    def __eq__(self, other):
        return libcrypto.X509_NAME_cmp(self.ptr, other.ptr) == 0

    def __getitem__(self, key):
        if isinstance(key, Oid):
            # Return first matching field
            idx = libcrypto.X509_NAME_get_index_by_NID(self.ptr, key.nid, -1)
            if idx < 0:
                raise KeyError("Key not found " + str(Oid))
            entry = libcrypto.X509_NAME_get_entry(self.ptr, idx)
            value = libcrypto.X509_NAME_ENTRY_get_data(entry)
            bio = Membio()
            libcrypto.ASN1_STRING_print_ex(bio.bio, value, self.PRINT_FLAG)
            return unicode(bio)
        elif isinstance(key, (int, long)):
            # Return OID, string tuple
            entry = libcrypto.X509_NAME_get_entry(self.ptr, key)
            if entry is None:
                raise IndexError("name entry index out of range")
            oid = Oid.fromobj(libcrypto.X509_NAME_ENTRY_get_object(entry))
            value = libcrypto.X509_NAME_ENTRY_get_data(entry)
            bio = Membio()
            libcrypto.ASN1_STRING_print_ex(bio.bio, value, self.PRINT_FLAG)
            return (oid, unicode(bio))
        else:
            raise TypeError("X509 NAME can be indexed by Oids or integers only")

    def __setitem__(self, key, val):
        if not self.writable:
            raise ValueError("Attempt to modify constant X509 object")
        else:
            raise NotImplementedError
    def __delitem__(self, key):
        if not self.writable:
            raise ValueError("Attempt to modify constant X509 object")
        else:
            raise NotImplementedError
    def __hash__(self):
        return libcrypto.X509_NAME_hash(self.ptr)

class _x509_ext(Structure):
    """ Represens C structure X509_EXTENSION """
    _fields_ = [("object", c_void_p),
                ("critical", c_int),
                ("value", c_void_p)
               ]

class X509_EXT(object):
    """ Python object which represents a certificate extension """
    def __init__(self, ptr, copy=False):
        """ Initializes from the pointer to X509_EXTENSION.
            If copy is True, creates a copy, otherwise just
            stores pointer.
        """
        if copy:
            self.ptr = libcrypto.X509_EXTENSION_dup(ptr)
        else:
            self.ptr = cast(ptr, POINTER(_x509_ext))
    def __del__(self):
        libcrypto.X509_EXTENSION_free(self.ptr)
    def __str__(self):
        bio = Membio()
        libcrypto.X509V3_EXT_print(bio.bio, self.ptr, 0x20010, 0)
        return str(bio)
    def __unicode__(self):
        bio = Membio()
        libcrypto.X509V3_EXT_print(bio.bio, self.ptr, 0x20010, 0)
        return unicode(bio)
    @property
    def oid(self):
        "Returns OID of the extension"
        return Oid.fromobj(self.ptr[0].object)
    @property
    def critical(self):
        "Returns True if extensin have critical flag set"
        return self.ptr[0].critical > 0

class _X509extlist(object):
    """
    Represents list of certificate extensions. Really it keeps
    reference to certificate object
    """
    def __init__(self, cert):
        """
        Initialize from X509 object
        """
        self.cert = cert

    def __len__(self):
        """
        Returns number of extensions
        """
        return libcrypto.X509_get_ext_count(self.cert.cert)

    def __getitem__(self, item):
        """
        Returns extension by index, creating a copy
        """
        ext_ptr = libcrypto.X509_get_ext(self.cert.cert, item)
        if ext_ptr is None:
            raise IndexError
        return X509_EXT(ext_ptr, True)
    def find(self, oid):
        """
        Return list of extensions with given Oid
        """
        if not isinstance(oid, Oid):
            raise TypeError("Need crytypescrypto.oid.Oid as argument")
        found = []
        index = -1
        end = len(self)
        while True:
            index = libcrypto.X509_get_ext_by_NID(self.cert.cert, oid.nid,
                                                  index)
            if index >= end or index < 0:
                break
            found.append(self[index])
        return found

    def find_critical(self, crit=True):
        """
        Return list of critical extensions (or list of non-cricital, if
        optional second argument is False
        """
        if crit:
            flag = 1
        else:
            flag = 0
        found = []
        end = len(self)
        index = -1
        while True:
            index = libcrypto.X509_get_ext_by_critical(self.cert.cert, flag,
                                                       index)
            if index >= end or index < 0:
                break
            found.append(self[index])
        return found

def _X509__asn1date_to_datetime(asn1date):
    """ 
    Converts openssl ASN1_TIME object to python datetime.datetime
    """
    bio = Membio()
    libcrypto.ASN1_TIME_print(bio.bio, asn1date)
    pydate = datetime.strptime(str(bio), "%b %d %H:%M:%S %Y %Z")
    return pydate.replace(tzinfo=utc)

class X509(object):
    """
    Represents X.509 certificate.
    """
    def __init__(self, data=None, ptr=None, format="PEM"):
        """
        Initializes certificate
        @param data - serialized certificate in PEM or DER format.
        @param ptr - pointer to X509, returned by some openssl function.
            mutually exclusive with data
        @param format - specifies data format. "PEM" or "DER", default PEM
        """
        if ptr is not None:
            if data is not None:
                raise TypeError("Cannot use data and ptr simultaneously")
            self.cert = ptr
        elif data is None:
            raise TypeError("data argument is required")
        else:
            bio = Membio(data)
            if format == "PEM":
                self.cert = libcrypto.PEM_read_bio_X509(bio.bio, None, None,
                                                        None)
            else:
                self.cert = libcrypto.d2i_X509_bio(bio.bio, None)
            if self.cert is None:
                raise X509Error("error reading certificate")
        self.extensions = _X509extlist(self)
    def __del__(self):
        """
        Frees certificate object
        """
        libcrypto.X509_free(self.cert)
    def __str__(self):
        """ Returns der string of the certificate """
        bio = Membio()
        if libcrypto.i2d_X509_bio(bio.bio, self.cert) == 0:
            raise X509Error("error serializing certificate")
        return str(bio)
    def __repr__(self):
        """ Returns valid call to the constructor """
        return "X509(data=" + repr(str(self)) + ",format='DER')"
    @property
    def pubkey(self):
        """EVP PKEy object of certificate public key"""
        return PKey(ptr=libcrypto.X509_get_pubkey(self.cert, False))
    def pem(self):
        """ Returns PEM represntation of the certificate """
        bio = Membio()
        if libcrypto.PEM_write_bio_X509(bio.bio, self.cert) == 0:
            raise X509Error("error serializing certificate")
        return str(bio)
    def verify(self, store=None, chain=None, key=None):
        """
        Verify self. Supports verification on both X509 store object
        or just public issuer key
        @param store X509Store object.
        @param chain - list of X509 objects to add into verification
            context.These objects are untrusted, but can be used to
            build certificate chain up to trusted object in the store
        @param key - PKey object with open key to validate signature

        parameters store and key are mutually exclusive. If neither
        is specified, attempts to verify self as self-signed certificate
        """
        if store is not None and key is not None:
            raise X509Error("key and store cannot be specified simultaneously")
        if store is not None:
            ctx = libcrypto.X509_STORE_CTX_new()
            if ctx is None:
                raise X509Error("Error allocating X509_STORE_CTX")
            if chain is not None and len(chain) > 0:
                chain_ptr = StackOfX509(chain).ptr
            else:
                chain_ptr = None
            if libcrypto.X509_STORE_CTX_init(ctx, store.store, self.cert,
                                             chain_ptr) < 0:
                raise X509Error("Error allocating X509_STORE_CTX")
            res = libcrypto.X509_verify_cert(ctx)
            libcrypto.X509_STORE_CTX_free(ctx)
            return res > 0
        else:
            if key is None:
                if self.issuer != self.subject:
                    # Not a self-signed certificate
                    return False
                key = self.pubkey
            res = libcrypto.X509_verify(self.cert, key.key)
            if res < 0:
                raise X509Error("X509_verify failed")
            return res > 0

    @property
    def subject(self):
        """ X509Name for certificate subject name """
        return X509Name(libcrypto.X509_get_subject_name(self.cert))
    @property
    def issuer(self):
        """ X509Name for certificate issuer name """
        return X509Name(libcrypto.X509_get_issuer_name(self.cert))
    @property
    def serial(self):
        """ Serial number of certificate as integer """
        asnint = libcrypto.X509_get_serialNumber(self.cert)
        bio = Membio()
        libcrypto.i2a_ASN1_INTEGER(bio.bio, asnint)
        return int(str(bio), 16)
    @property
    def version(self):
        """
        certificate version as integer. Really certificate stores 0 for
        version 1 and 2 for version 3, but we return 1 and 3
        """
        return _X509_get_version(self.cert) + 1
    @property
    def startDate(self):
        """ Certificate validity period start date """
        asn1 = _X509_get_notBefore(self.cert)
        return __asn1date_to_datetime(asn1)
    @property
    def endDate(self):
        """ Certificate validity period end date """
        asn1 = _X509_get_notAfter(self.cert)
        return __asn1date_to_datetime(asn1)
    def check_ca(self):
        """ Returns True if certificate is CA certificate """
        return libcrypto.X509_check_ca(self.cert) > 0

class X509Store(object):
    """
    Represents trusted certificate store. Can be used to lookup CA
    certificates to verify

    @param file - file with several certificates and crls
            to load into store
    @param dir - hashed directory with certificates and crls
    @param default - if true, default verify location (directory)
        is installed

    """
    def __init__(self, file=None, dir=None, default=False):
        """
        Creates X509 store and installs lookup method. Optionally initializes
        by certificates from given file or directory.
        """
        #
        # Todo - set verification flags
        #
        self.store = libcrypto.X509_STORE_new()
        if self.store is None:
            raise X509Error("allocating store")
        lookup = libcrypto.X509_STORE_add_lookup(self.store,
                                                 libcrypto.X509_LOOKUP_file())
        if lookup is None:
            raise X509Error("error installing file lookup method")
        if file is not None:
            if not libcrypto.X509_LOOKUP_ctrl(lookup, 1, file, 1, None) > 0:
                raise X509Error("error loading trusted certs from file "+file)
        lookup = libcrypto.X509_STORE_add_lookup(self.store,
                                             libcrypto.X509_LOOKUP_hash_dir())
        if lookup is None:
            raise X509Error("error installing hashed lookup method")
        if dir is not None:
            if not libcrypto.X509_LOOKUP_ctrl(lookup, 2, dir, 1, None) > 0:
                raise X509Error("error adding hashed  trusted certs dir "+dir)
        if default:
            if not libcrypto.X509_LOOKUP_ctrl(lookup, 2, None, 3, None) > 0:
                raise X509Error("error adding default trusted certs dir ")
    def add_cert(self, cert):
        """
        Explicitely adds certificate to set of trusted in the store
        @param cert - X509 object to add
        """
        if not isinstance(cert, X509):
            raise TypeError("cert should be X509")
        libcrypto.X509_STORE_add_cert(self.store, cert.cert)
    def add_callback(self, callback):
        """
        Installs callback function, which would receive detailed information
        about verified ceritificates
        """
        raise NotImplementedError
    def setflags(self, flags):
        """
        Set certificate verification flags.
        @param flags - integer bit mask. See OpenSSL X509_V_FLAG_* constants
        """
        libcrypto.X509_STORE_set_flags(self.store, flags)
    def setpurpose(self, purpose):
        """
        Sets certificate purpose which verified certificate should match
        @param purpose - number from 1 to 9 or standard strind defined
                         in Openssl
        possible strings - sslcient,sslserver, nssslserver, smimesign,i
                         smimeencrypt, crlsign, any, ocsphelper
        """
        if isinstance(purpose, str):
            purp_no = libcrypto.X509_PURPOSE_get_by_sname(purpose)
            if purp_no <= 0:
                raise X509Error("Invalid certificate purpose '%s'" % purpose)
        elif isinstance(purpose, int):
            purp_no = purpose
        if libcrypto.X509_STORE_set_purpose(self.store, purp_no) <= 0:
            raise X509Error("cannot set purpose")
    def setdepth(self, depth):
        """
        Sets the verification depth i.e. max length of certificate chain
        which is acceptable
        """
        libcrypto.X509_STORE_set_depth(self.store, depth)
    def settime(self, time):
        """
        Set point in time used to check validity of certificates for
        Time can be either python datetime object or number of seconds
        sinse epoch
        """
        if isinstance(time, datetime) or isinstance(time,
                                                             datetime.date):
            seconds = int(time.strftime("%s"))
        elif isinstance(time, int):
            seconds = time
        else:
            raise TypeError("datetime.date, datetime.datetime or integer " +
                            "is required as time argument")
        raise NotImplementedError
class StackOfX509(object):
    """
    Implements OpenSSL STACK_OF(X509) object.
    It looks much like python container types
    """
    def __init__(self, certs=None, ptr=None, disposable=True):
        """
        Create stack
        @param certs - list of X509 objects. If specified, read-write
            stack is created and populated by these certificates
        @param ptr - pointer to OpenSSL STACK_OF(X509) as returned by
            some functions
        @param disposable - if True, stack created from object, returned
                by function is copy, and can be modified and need to be
                freeid. If false, it is just pointer into another
                structure i.e. CMS_ContentInfo
        """
        self.need_free = False
        if  ptr is None:
            self.need_free = True
            self.ptr = libcrypto.sk_new_null()
            if certs is not None:
                for crt in certs:
                    self.append(crt)
        elif certs is not None:
            raise ValueError("cannot handle certs an ptr simultaneously")
        else:
            self.need_free = disposable
            self.ptr = ptr
    def __len__(self):
        return libcrypto.sk_num(self.ptr)
    def __getitem__(self, index):
        if index < 0 or index >= len(self):
            raise IndexError
        p = libcrypto.sk_value(self.ptr, index)
        return X509(ptr=libcrypto.X509_dup(p))
    def __setitem__(self, index, value):
        if not self.need_free:
            raise ValueError("Stack is read-only")
        if index < 0 or index >= len(self):
            raise IndexError
        if not isinstance(value, X509):
            raise TypeError('StackOfX508 can contain only X509 objects')
        p = libcrypto.sk_value(self.ptr, index)
        libcrypto.sk_set(self.ptr, index, libcrypto.X509_dup(value.cert))
        libcrypto.X509_free(p)
    def __delitem__(self, index):
        if not self.need_free:
            raise ValueError("Stack is read-only")
        if index < 0 or index >= len(self):
            raise IndexError
        p = libcrypto.sk_delete(self.ptr, index)
        libcrypto.X509_free(p)
    def __del__(self):
        if self.need_free:
            libcrypto.sk_pop_free(self.ptr, libcrypto.X509_free)
    def append(self, value):
        """ Adds certificate to stack """
        if not self.need_free:
            raise ValueError("Stack is read-only")
        if not isinstance(value, X509):
            raise TypeError('StackOfX508 can contain only X509 objects')
        libcrypto.sk_push(self.ptr, libcrypto.X509_dup(value.cert))

libcrypto.i2a_ASN1_INTEGER.argtypes = (c_void_p, c_void_p)
libcrypto.ASN1_STRING_print_ex.argtypes = (c_void_p, c_void_p, c_long)
libcrypto.PEM_read_bio_X509.restype = c_void_p
libcrypto.PEM_read_bio_X509.argtypes = (c_void_p, POINTER(c_void_p),
                                        c_void_p, c_void_p)
libcrypto.PEM_write_bio_X509.restype = c_int
libcrypto.PEM_write_bio_X509.argtypes = (c_void_p, c_void_p)
libcrypto.ASN1_TIME_print.argtypes = (c_void_p, c_void_p)
libcrypto.ASN1_INTEGER_get.argtypes = (c_void_p, )
libcrypto.ASN1_INTEGER_get.restype = c_long
libcrypto.X509_get_serialNumber.argtypes = (c_void_p, )
libcrypto.X509_get_serialNumber.restype = c_void_p
libcrypto.X509_NAME_ENTRY_get_object.restype = c_void_p
libcrypto.X509_NAME_ENTRY_get_object.argtypes = (c_void_p, )
libcrypto.OBJ_obj2nid.argtypes = (c_void_p, )
libcrypto.X509_NAME_get_entry.restype = c_void_p
libcrypto.X509_NAME_get_entry.argtypes = (c_void_p, c_int)
libcrypto.X509_STORE_new.restype = c_void_p
libcrypto.X509_STORE_add_lookup.restype = c_void_p
libcrypto.X509_STORE_add_lookup.argtypes = (c_void_p, c_void_p)
libcrypto.X509_LOOKUP_file.restype = c_void_p
libcrypto.X509_LOOKUP_hash_dir.restype = c_void_p
libcrypto.X509_LOOKUP_ctrl.restype = c_int
libcrypto.X509_LOOKUP_ctrl.argtypes = (c_void_p, c_int, c_char_p, c_long,
                                       POINTER(c_char_p))
libcrypto.X509_EXTENSION_dup.argtypes = (c_void_p, )
libcrypto.X509_EXTENSION_dup.restype = POINTER(_x509_ext)
libcrypto.X509V3_EXT_print.argtypes = (c_void_p, POINTER(_x509_ext), c_long,
                                       c_int)
libcrypto.X509_get_ext.restype = c_void_p
libcrypto.X509_get_ext.argtypes = (c_void_p, c_int)
libcrypto.X509V3_EXT_print.argtypes = (c_void_p, POINTER(_x509_ext), c_long,
                                       c_int)
libcrypto.sk_set.argtypes = (c_void_p, c_int, c_void_p)
libcrypto.sk_set.restype = c_void_p
libcrypto.sk_value.argtypes = (c_void_p, c_int)
libcrypto.sk_value.restype = c_void_p
libcrypto.X509_dup.restype = c_void_p
libcrypto.sk_new_null.restype = c_void_p
libcrypto.X509_dup.argtypes = (c_void_p, )
libcrypto.X509_NAME_hash.restype = c_long
libcrypto.X509_NAME_hash.argtypes = (c_void_p, )



from ctypescrypto.oid import Oid,create,cleanup
import unittest

class TestStandard(unittest.TestCase):
    def test_cn(self):
        o=Oid("2.5.4.3")
        self.assertEqual(repr(o),"Oid('2.5.4.3')")
        self.assertEqual(o.dotted(),"2.5.4.3")
        self.assertEqual(str(o),"2.5.4.3")
        self.assertEqual(o.shortname(),"CN")
        self.assertEqual(o.longname(),"commonName")
    def test_getnid(self):
        o=Oid("2.5.4.3")
        x=Oid("CN")
        self.assertEqual(o.nid,x.nid)
        self.assertEqual(o,x)
        self.assertEqual(hash(o),hash(x))

    def test_cons2(self):
        o=Oid("2.5.4.3")
        x=Oid("commonName")
        self.assertEqual(o.nid,x.nid)
    def test_bynid(self):
        o=Oid("2.5.4.3")
        x=Oid(o.nid)
        self.assertEqual(o.nid,x.nid)
    def test_fromunicode(self):
        o=Oid(u'commonName')
        self.assertEqual(o.shortname(),'CN')
    def test_wrongoid(self):
        with self.assertRaises(ValueError):
            o=Oid("1.2.3.4.5.6.7.8.10.111.1111")
    def test_wrongname(self):
        with self.assertRaises(ValueError):
            o=Oid("No such oid in the database")
    def test_wrongnid(self):
        with self.assertRaises(ValueError):
            o=Oid(9999999)
    def test_wrongtype(self):
        with self.assertRaises(TypeError):
            o=Oid([2,5,3,4])

class TestCustom(unittest.TestCase):
    def testCreate(self):
        d='1.2.643.100.3'
        sn="SNILS"
        long_name="Russian Pension security number"
        o=create(d,sn,long_name)
        self.assertEqual(str(o),d)
        self.assertEqual(o.shortname(),sn)
        self.assertEqual(o.longname(),long_name)
    def testLookup(self):
        d='1.2.643.100.3'
        sn="SNILS"
        long_name="Russian Pension security number"
        o=create(d,sn,long_name)
        x=Oid(sn)
        self.assertEqual(o,x)
    def testCleanup(self):
        d='1.2.643.100.9'
        sn="SNILX"
        long_name="Russian Pension security number"
        o=create(d,sn,long_name)
        cleanup()
        with self.assertRaises(ValueError):
            x=Oid(sn)
    def tearDown(self):
        # Always call cleanup before next test
        cleanup()
    



if __name__ == '__main__':
    unittest.main()



"""
PKCS5 PBKDF2 function.

"""

from ctypes import c_char_p, c_int, c_void_p, create_string_buffer
from ctypescrypto import libcrypto
from ctypescrypto.digest import DigestType
from ctypescrypto.exception import LibCryptoError

__all__ = ['pbkdf2']

def pbkdf2(password, salt, outlen, digesttype="sha1", iterations=2000):
    """
    Interface to PKCS5_PBKDF2_HMAC function
    Parameters:

    @param password - password to derive key from
    @param salt - random salt to use for key derivation
    @param outlen - number of bytes to derive
    @param digesttype - name of digest to use to use (default sha1)
    @param iterations - number of iterations to use

    @returns outlen bytes of key material derived from password and salt
    """
    dgst = DigestType(digesttype)
    out = create_string_buffer(outlen)
    res = libcrypto.PKCS5_PBKDF2_HMAC(password, len(password), salt, len(salt),
                                      iterations, dgst.digest, outlen, out)
    if res <= 0:
        raise LibCryptoError("error computing PBKDF2")
    return out.raw

libcrypto.PKCS5_PBKDF2_HMAC.argtypes = (c_char_p, c_int, c_char_p, c_int, c_int,
                                        c_void_p, c_int, c_char_p)
libcrypto.PKCS5_PBKDF2_HMAC.restupe = c_int



"""
This module provides interface for low-level private/public keypair operation

PKey object of this module is wrapper around OpenSSL EVP_PKEY object.
"""


from ctypes import c_char_p, c_void_p, c_int, c_long, POINTER
from ctypes import create_string_buffer, byref, memmove, CFUNCTYPE
from ctypescrypto import libcrypto
from ctypescrypto.exception import LibCryptoError, clear_err_stack
from ctypescrypto.bio import Membio

__all__ = ['PKeyError', 'password_callback', 'PKey', 'PW_CALLBACK_FUNC']
class PKeyError(LibCryptoError):
    """ Exception thrown if libcrypto finctions return an error """
    pass

PW_CALLBACK_FUNC = CFUNCTYPE(c_int, c_char_p, c_int, c_int, c_char_p)
""" Function type for pem password callback """

def password_callback(buf, length, rwflag, userdata):
    """
    Example password callback for private key. Assumes that
    password is stored in the userdata parameter, so allows to pass password
    from constructor arguments to the libcrypto keyloading functions
    """
    cnt = len(userdata)
    if length < cnt:
        cnt = length
    memmove(buf, userdata, cnt)
    return cnt

_cb = PW_CALLBACK_FUNC(password_callback)

class PKey(object):
    """
    Represents public/private key pair. Wrapper around EVP_PKEY
    libcrypto object.

    May contain either both private and public key (such objects can be
    used for signing, deriving shared key as well as verifying or public
    key only, which can be used for verifying or as peer key when
    deriving.

    @var cansign is true key has private part.
    @var key contain pointer to EVP_PKEY and should be passed to various
         libcrypto routines
    """
    def __init__(self, ptr=None, privkey=None, pubkey=None, format="PEM",
                 cansign=False, password=None, callback=_cb):
        if not ptr is None:
            self.key = ptr
            self.cansign = cansign
            if not privkey is None or not pubkey is None:
                raise TypeError("Just one of ptr, pubkey or privkey can " +
                                "be specified")
        elif not privkey is None:
            if not pubkey is None:
                raise TypeError("Just one of ptr, pubkey or privkey can " +
                                "be specified")
            bio = Membio(privkey)
            self.cansign = True
            if format == "PEM":
                self.key = libcrypto.PEM_read_bio_PrivateKey(bio.bio, None,
                                                             callback,
                                                             c_char_p(password))
            else:
                self.key = libcrypto.d2i_PrivateKey_bio(bio.bio, None)
            if self.key is None:
                raise PKeyError("error parsing private key")
        elif not pubkey is None:
            bio = Membio(pubkey)
            self.cansign = False
            if format == "PEM":
                self.key = libcrypto.PEM_read_bio_PUBKEY(bio.bio, None,
                                                         callback,
                                                         c_char_p(password))
            else:
                self.key = libcrypto.d2i_PUBKEY_bio(bio.bio, None)
            if self.key is None:
                raise PKeyError("error parsing public key")
        else:
            raise TypeError("Neither public, nor private key is specified")


    def __del__(self):
        """ Frees EVP_PKEY object (note, it is reference counted) """
        libcrypto.EVP_PKEY_free(self.key)

    def __eq__(self, other):
        """ Compares two public keys. If one has private key and other
            doesn't it doesn't affect result of comparation
        """
        return libcrypto.EVP_PKEY_cmp(self.key, other.key) == 1

    def __ne__(self, other):
        """ Compares two public key for not-equality """
        return not self.__eq__(other)

    def __str__(self):
        """ printable representation of public key """
        bio = Membio()
        libcrypto.EVP_PKEY_print_public(bio.bio, self.key, 0, None)
        return str(bio)

    def sign(self, digest, **kwargs):
        """
        Signs given digest and retirns signature
        Keyword arguments allows to set various algorithm-specific
        parameters. See pkeyutl(1) manual.
        """
        ctx = libcrypto.EVP_PKEY_CTX_new(self.key, None)
        if ctx is None:
            raise PKeyError("Initailizing sign context")
        if libcrypto.EVP_PKEY_sign_init(ctx) < 1:
            raise PKeyError("sign_init")
        self._configure_context(ctx, kwargs)
        # Find out signature size
        siglen = c_long(0)
        if libcrypto.EVP_PKEY_sign(ctx, None, byref(siglen), digest,
                                   len(digest)) < 1:
            raise PKeyError("computing signature length")
        sig = create_string_buffer(siglen.value)
        if libcrypto.EVP_PKEY_sign(ctx, sig, byref(siglen), digest,
                                   len(digest)) < 1:
            raise PKeyError("signing")
        libcrypto.EVP_PKEY_CTX_free(ctx)
        return sig.raw[:int(siglen.value)]

    def verify(self, digest, signature, **kwargs):
        """
        Verifies given signature on given digest
        Returns True if Ok, False if don't match
        Keyword arguments allows to set algorithm-specific
        parameters
        """
        ctx = libcrypto.EVP_PKEY_CTX_new(self.key, None)
        if ctx is None:
            raise PKeyError("Initailizing verify context")
        if libcrypto.EVP_PKEY_verify_init(ctx) < 1:
            raise PKeyError("verify_init")
        self._configure_context(ctx, kwargs)
        ret = libcrypto.EVP_PKEY_verify(ctx, signature, len(signature), digest,
                                        len(digest))
        if ret < 0:
            raise PKeyError("Signature verification")
        libcrypto.EVP_PKEY_CTX_free(ctx)
        return ret > 0

    def derive(self, peerkey, **kwargs):
        """
        Derives shared key (DH,ECDH,VKO 34.10). Requires
        private key available

        @param peerkey - other key (may be public only)

        Keyword parameters are algorithm-specific
        """
        if not self.cansign:
            raise ValueError("No private key available")
        ctx = libcrypto.EVP_PKEY_CTX_new(self.key, None)
        if ctx is None:
            raise PKeyError("Initailizing derive context")
        if libcrypto.EVP_PKEY_derive_init(ctx) < 1:
            raise PKeyError("derive_init")

        # This is workaround around missing functionality in GOST engine
        # it provides only numeric control command to set UKM, not
        # string one.
        self._configure_context(ctx, kwargs, ["ukm"])
        if libcrypto.EVP_PKEY_derive_set_peer(ctx, peerkey.key) <= 0:
            raise PKeyError("Cannot set peer key")
        if "ukm" in kwargs:
            # We just hardcode numeric command to set UKM here
            if libcrypto.EVP_PKEY_CTX_ctrl(ctx, -1, 1 << 10, 8, 8,
                                           kwargs["ukm"]) <= 0:
                raise PKeyError("Cannot set UKM")
        keylen = c_long(0)
        if libcrypto.EVP_PKEY_derive(ctx, None, byref(keylen)) <= 0:
            raise PKeyError("computing shared key length")
        buf = create_string_buffer(keylen.value)
        if libcrypto.EVP_PKEY_derive(ctx, buf, byref(keylen)) <= 0:
            raise PKeyError("computing actual shared key")
        libcrypto.EVP_PKEY_CTX_free(ctx)
        return buf.raw[:int(keylen.value)]

    @staticmethod
    def generate(algorithm, **kwargs):
        """
        Generates new private-public key pair for given algorithm
        (string like 'rsa','ec','gost2001') and algorithm-specific
        parameters.

        Algorithm specific paramteers for RSA:

        rsa_keygen_bits=number - size of key to be generated
        rsa_keygen_pubexp - RSA public expontent(default 65537)

        Algorithm specific parameters for DSA,DH and EC

        paramsfrom=PKey object

        copy parameters of newly generated key from existing key

        Algorithm specific parameters for GOST2001

        paramset= paramset name where name is one of
        'A','B','C','XA','XB','test'

        paramsfrom does work too
        """
        tmpeng = c_void_p(None)
        ameth = libcrypto.EVP_PKEY_asn1_find_str(byref(tmpeng), algorithm, -1)
        if ameth is None:
            raise PKeyError("Algorithm %s not foind\n"%(algorithm))
        clear_err_stack()
        pkey_id = c_int(0)
        libcrypto.EVP_PKEY_asn1_get0_info(byref(pkey_id), None, None, None,
                                          None, ameth)
        #libcrypto.ENGINE_finish(tmpeng)
        if "paramsfrom" in kwargs:
            ctx = libcrypto.EVP_PKEY_CTX_new(kwargs["paramsfrom"].key, None)
        else:
            ctx = libcrypto.EVP_PKEY_CTX_new_id(pkey_id, None)
        # FIXME support EC curve as keyword param by invoking paramgen
        # operation
        if ctx is None:
            raise PKeyError("Creating context for key type %d"%(pkey_id.value))
        if libcrypto.EVP_PKEY_keygen_init(ctx) <= 0:
            raise PKeyError("keygen_init")
        PKey._configure_context(ctx, kwargs, ["paramsfrom"])
        key = c_void_p(None)
        if libcrypto.EVP_PKEY_keygen(ctx, byref(key)) <= 0:
            raise PKeyError("Error generating key")
        libcrypto.EVP_PKEY_CTX_free(ctx)
        return PKey(ptr=key, cansign=True)

    def exportpub(self, format="PEM"):
        """
        Returns public key as PEM or DER structure.
        """
        bio = Membio()
        if format == "PEM":
            retcode = libcrypto.PEM_write_bio_PUBKEY(bio.bio, self.key)
        else:
            retcode = libcrypto.i2d_PUBKEY_bio(bio.bio, self.key)
        if retcode == 0:
            raise PKeyError("error serializing public key")
        return str(bio)

    def exportpriv(self, format="PEM", password=None, cipher=None,
                   callback=_cb):
        """
        Returns private key as PEM or DER Structure.
        If password and cipher are specified, encrypts key
        on given password, using given algorithm. Cipher must be
        an ctypescrypto.cipher.CipherType object
        """
        bio = Membio()
        if cipher is None:
            evp_cipher = None
        else:
            evp_cipher = cipher.cipher
        if format == "PEM":
            ret = libcrypto.PEM_write_bio_PrivateKey(bio.bio, self.key,
                                                     evp_cipher, None, 0,
                                                     callback,
                                                     c_char_p(password))
        else:
            ret = libcrypto.i2d_PKCS8PrivateKey_bio(bio.bio, self.key,
                                                    evp_cipher, None, 0,
                                                    callback,
                                                    c_char_p(password))
        if ret == 0:
            raise PKeyError("error serializing private key")
        return str(bio)

    @staticmethod
    def _configure_context(ctx, opts, skip=()):
        """
        Configures context of public key operations
        @param ctx - context to configure
        @param opts - dictionary of options (from kwargs of calling
            function)
        @param skip - list of options which shouldn't be passed to
            context
        """

        for oper in opts:
            if oper in skip:
                continue
            ret = libcrypto.EVP_PKEY_CTX_ctrl_str(ctx, oper, str(opts[oper]))
            if ret == -2:
                raise PKeyError("Parameter %s is not supported by key" % oper)
            if ret < 1:
                raise PKeyError("Error setting parameter %s" % oper)
# Declare function prototypes
libcrypto.EVP_PKEY_cmp.argtypes = (c_void_p, c_void_p)
libcrypto.PEM_read_bio_PrivateKey.restype = c_void_p
libcrypto.PEM_read_bio_PrivateKey.argtypes = (c_void_p, POINTER(c_void_p),
                                              PW_CALLBACK_FUNC, c_char_p)
libcrypto.PEM_read_bio_PUBKEY.restype = c_void_p
libcrypto.PEM_read_bio_PUBKEY.argtypes = (c_void_p, POINTER(c_void_p),
                                          PW_CALLBACK_FUNC, c_char_p)
libcrypto.d2i_PUBKEY_bio.restype = c_void_p
libcrypto.d2i_PUBKEY_bio.argtypes = (c_void_p, c_void_p)
libcrypto.d2i_PrivateKey_bio.restype = c_void_p
libcrypto.d2i_PrivateKey_bio.argtypes = (c_void_p, c_void_p)
libcrypto.EVP_PKEY_print_public.argtypes = (c_void_p, c_void_p, c_int, c_void_p)
libcrypto.EVP_PKEY_asn1_find_str.restype = c_void_p
libcrypto.EVP_PKEY_asn1_find_str.argtypes = (c_void_p, c_char_p, c_int)
libcrypto.EVP_PKEY_asn1_get0_info.restype = c_int
libcrypto.EVP_PKEY_asn1_get0_info.argtypes = (POINTER(c_int), POINTER(c_int),
                                              POINTER(c_int), POINTER(c_char_p),
                                              POINTER(c_char_p), c_void_p)
libcrypto.EVP_PKEY_cmp.restype = c_int
libcrypto.EVP_PKEY_cmp.argtypes = (c_void_p, c_void_p)
libcrypto.EVP_PKEY_CTX_ctrl_str.restype = c_int
libcrypto.EVP_PKEY_CTX_ctrl_str.argtypes = (c_void_p, c_void_p, c_void_p)
libcrypto.EVP_PKEY_CTX_ctrl.restype = c_int
libcrypto.EVP_PKEY_CTX_ctrl.argtypes = (c_void_p, c_int, c_int, c_int, c_int,
                                        c_void_p)
libcrypto.EVP_PKEY_CTX_free.argtypes = (c_void_p, )
libcrypto.EVP_PKEY_CTX_new.restype = c_void_p
libcrypto.EVP_PKEY_CTX_new.argtypes = (c_void_p, c_void_p)
libcrypto.EVP_PKEY_CTX_new_id.restype = c_void_p
libcrypto.EVP_PKEY_CTX_new_id.argtypes = (c_int, c_void_p)
libcrypto.EVP_PKEY_derive.restype = c_int
libcrypto.EVP_PKEY_derive.argtypes = (c_void_p, c_char_p, POINTER(c_long))
libcrypto.EVP_PKEY_derive_init.restype = c_int
libcrypto.EVP_PKEY_derive_init.argtypes = (c_void_p, )
libcrypto.EVP_PKEY_derive_set_peer.restype = c_int
libcrypto.EVP_PKEY_derive_set_peer.argtypes = (c_void_p, c_void_p)
libcrypto.EVP_PKEY_free.argtypes = (c_void_p,)
libcrypto.EVP_PKEY_keygen.restype = c_int
libcrypto.EVP_PKEY_keygen.argtypes = (c_void_p, c_void_p)
libcrypto.EVP_PKEY_keygen_init.restype = c_int
libcrypto.EVP_PKEY_keygen_init.argtypes = (c_void_p, )
libcrypto.EVP_PKEY_sign.restype = c_int
libcrypto.EVP_PKEY_sign.argtypes = (c_void_p, c_char_p, POINTER(c_long),
                                    c_char_p, c_long)
libcrypto.EVP_PKEY_sign_init.restype = c_int
libcrypto.EVP_PKEY_sign_init.argtypes = (c_void_p, )
libcrypto.EVP_PKEY_verify.restype = c_int
libcrypto.EVP_PKEY_verify.argtypes = (c_void_p, c_char_p, c_long, c_char_p,
                                      c_long)
libcrypto.EVP_PKEY_verify_init.restype = c_int
libcrypto.EVP_PKEY_verify_init.argtypes = (c_void_p, )
libcrypto.PEM_write_bio_PrivateKey.argtypes = (c_void_p, c_void_p, c_void_p,
                                               c_char_p, c_int,
                                               PW_CALLBACK_FUNC, c_char_p)
libcrypto.PEM_write_bio_PUBKEY.argtypes = (c_void_p, c_void_p)
libcrypto.i2d_PUBKEY_bio.argtypes = (c_void_p, c_void_p)
libcrypto.i2d_PKCS8PrivateKey_bio.argtypes = (c_void_p, c_void_p, c_void_p,
                                              c_char_p, c_int,
                                              PW_CALLBACK_FUNC, c_char_p)
libcrypto.ENGINE_finish.argtypes = (c_void_p, )



from ctypescrypto.oid import Oid
from ctypescrypto import cipher
import unittest
import sys


class TestCipherType(unittest.TestCase):
    def test_ciphdescbc(self):
        ct=cipher.CipherType("des-cbc")
        self.assertEqual(ct.block_size(),8)
        self.assertEqual(ct.key_length(),8)
        self.assertEqual(ct.iv_length(),8)
        self.assertEqual(ct.algo(),'DES-CBC')
        self.assertEqual(ct.oid().shortname(),"DES-CBC")
        self.assertEqual(ct.mode(),"CBC")
    def test_ciphaesofb(self):
        ct=cipher.CipherType("aes-256-ofb")
        self.assertEqual(ct.block_size(),1)
        self.assertEqual(ct.key_length(),32)
        self.assertEqual(ct.iv_length(),16)
        self.assertEqual(ct.algo(),'AES-256-OFB')
        self.assertEqual(ct.oid().shortname(),"AES-256-OFB")
        self.assertEqual(ct.mode(),"OFB")
    def test_unknowncipher(self):
        with self.assertRaises(cipher.CipherError):
            ct=cipher.CipherType("no-such-cipher")

class TestEncryptDecrypt(unittest.TestCase):
    def test_cons_nokey(self):
        ct=cipher.CipherType("DES-CBC")
        with self.assertRaises(ValueError):
            c=cipher.Cipher(ct,None,None)
    def test_blockcipher(self):
        data="sdfdsddf"
        key='abcdabcd'
        c=cipher.new("DES-CBC",key)
        enc=c.update(data)+c.finish()
        # See if padding is added by default
        self.assertEqual(len(enc),16)
        d=cipher.new("DES-CBC",key,encrypt=False)
        dec=d.update(enc)+d.finish()
        self.assertEqual(data,dec)
    def test_blockcipher_nopadding(self):
        data="sdfdsddf"
        key='abcdabcd'
        c=cipher.new("DES-CBC",key)
        c.padding(False)
        enc=c.update(data)+c.finish()
        # See if padding is added by default
        self.assertEqual(len(enc),8)
        d=cipher.new("DES-CBC",key,encrypt=False)
        d.padding(False)
        dec=d.update(enc)+d.finish()
        self.assertEqual(data,dec)
    def test_ofb_cipher(self):
        data="sdfdsddfxx"
        key='abcdabcd'
        iv='abcdabcd'
        c=cipher.new("DES-OFB",key,iv=iv)
        enc=c.update(data)+c.finish()
        # See if padding is added by default
        self.assertEqual(len(enc),len(data))
        d=cipher.new("DES-OFB",key,encrypt=False,iv=iv)
        dec=d.update(enc)+d.finish()
        self.assertEqual(data,dec)

    def test_ofb_noiv(self):
        data="sdfdsddfxx"
        encryptkey='abcdabcd'*4
        decryptkey=encryptkey[0:5]+encryptkey[5:]


        c=cipher.new("AES-256-OFB",encryptkey)
        enc=c.update(data)+c.finish()
        # See if padding is added by default
        self.assertEqual(len(enc),len(data))
        d=cipher.new("AES-256-OFB",decryptkey,encrypt=False)
        dec=d.update(enc)+d.finish()
        self.assertEqual(data,dec)
    def test_wrong_keylength(self):
        data="sdfsdfxxx"
        key="abcdabcd"
        with self.assertRaises(ValueError):
            c=cipher.new("AES-128-OFB",key)
    def test_wrong_ivlength(self):
        key="abcdabcdabcdabcd"
        iv="xxxxx"
        with self.assertRaises(ValueError):
            c=cipher.new("AES-128-OFB",key,iv=iv)
    def test_variable_keylength(self):
        encryptkey="abcdefabcdefghtlgvasdgdgsdgdsg"
        data="asdfsdfsdfsdfsdfsdfsdfsdf"
        iv="abcdefgh"
        c=cipher.new("bf-ofb",encryptkey,iv=iv)
        ciphertext=c.update(data)+c.finish()
        decryptkey=encryptkey[0:5]+encryptkey[5:]
        d=cipher.new("bf-ofb",decryptkey,encrypt=False,iv=iv)
        deciph=d.update(ciphertext)+d.finish()
        self.assertEqual(deciph,data)

if __name__ == '__main__':
    unittest.main()



"""
    Interface to the OpenSSL pseudo-random generator
"""

from ctypes import create_string_buffer, c_char_p, c_int, c_double
from ctypescrypto import libcrypto
from ctypescrypto.exception import LibCryptoError

__all__ = ['RandError', 'bytes', 'pseudo_bytes', 'seed', 'status']

class RandError(LibCryptoError):
    """ Exception raised when openssl function return error """
    pass

def bytes(num, check_result=False):
    """
    Returns num bytes of cryptographically strong pseudo-random
    bytes. If checkc_result is True, raises error if PRNG is not
    seeded enough
    """

    if num <= 0:
        raise ValueError("'num' should be > 0")
    buf = create_string_buffer(num)
    result = libcrypto.RAND_bytes(buf, num)
    if check_result and result == 0:
        raise RandError("Random Number Generator not seeded sufficiently")
    return buf.raw[:num]

def pseudo_bytes(num):
    """
    Returns num bytes of pseudo random data.  Pseudo- random byte
    sequences generated by pseudo_bytes() will be unique if
    they are of sufficient length, but are not necessarily
    unpredictable. They can be used for non-cryptographic purposes
    and for certain purposes in cryptographic protocols, but usually
    not for key generation etc.
    """
    if num <= 0:
        raise ValueError("'num' should be > 0")
    buf = create_string_buffer(num)
    libcrypto.RAND_pseudo_bytes(buf, num)
    return buf.raw[:num]

def seed(data, entropy=None):
    """
        Seeds random generator with data.
        If entropy is not None, it should be floating point(double)
        value estimating amount of entropy  in the data (in bytes).
    """
    if not isinstance(data, str):
        raise TypeError("A string is expected")
    ptr = c_char_p(data)
    size = len(data)
    if entropy is None:
        libcrypto.RAND_seed(ptr, size)
    else:
        libcrypto.RAND_add(ptr, size, entropy)

def status():
    """
    Returns 1 if random generator is sufficiently seeded and 0
    otherwise
    """

    return libcrypto.RAND_status()

libcrypto.RAND_add.argtypes = (c_char_p, c_int, c_double)
libcrypto.RAND_seed.argtypes = (c_char_p, c_int)
libcrypto.RAND_pseudo_bytes.argtypes = (c_char_p, c_int)
libcrypto.RAND_bytes.argtypes = (c_char_p, c_int)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

from ctypescrypto.x509 import X509,X509Store,utc,StackOfX509
from ctypescrypto.oid import Oid
from tempfile import NamedTemporaryFile
import datetime
import unittest
import os



class TestCertInfo(unittest.TestCase):
    ca_cert="""-----BEGIN CERTIFICATE-----
MIIEDzCCAvegAwIBAgIJAN9Ejmna3JJ7MA0GCSqGSIb3DQEBBQUAMIGdMQswCQYD
VQQGEwJSVTEVMBMGA1UECAwM0JzQvtGB0LrQstCwMTAwLgYDVQQKDCfQo9C00L7R
gdGC0L7QstC10YDRj9GO0YnQuNC5INGG0LXQvdGC0YAxIjAgBgNVBAMMGdCS0LjQ
utGC0L7RgCDQktCw0LPQvdC10YAxITAfBgkqhkiG9w0BCQEWEnZpdHVzQHdhZ25l
ci5wcC5ydTAeFw0xNDEwMjYxNDQ2MzJaFw0xNzEwMjUxNDQ2MzJaMIGdMQswCQYD
VQQGEwJSVTEVMBMGA1UECAwM0JzQvtGB0LrQstCwMTAwLgYDVQQKDCfQo9C00L7R
gdGC0L7QstC10YDRj9GO0YnQuNC5INGG0LXQvdGC0YAxIjAgBgNVBAMMGdCS0LjQ
utGC0L7RgCDQktCw0LPQvdC10YAxITAfBgkqhkiG9w0BCQEWEnZpdHVzQHdhZ25l
ci5wcC5ydTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAJn+nL8CDaM0
KNafGYdEDuFuCKHFxCcbaT7ecGbwjPKtnqZLOnYpa2iLFY+n7zAYw1MRkFkaR8b+
+AeFPWS9T60ygeWysT9XTS77Fdl69Tmr8HChRk0BuLB3vFCy34vBHSG6Frdm8PtS
PLEleldiwUCHLS+EckrnJylQK13X3HofSbIGzKK53MsuQOtp2dJw3b7MILM/+XBm
RgZlEbTEPYMOH8CE3mu9/LqXfIRQM7+nmzcNZI3RAwxEVHOSHEbgFZaksTj8rMIa
SrJwknmxHntI3P5PSNNbs0SO3TW8ePDIIpbVcjNsMX4qGX8b+8quZuzciKOto8S0
0A6eOBd8Vi0CAwEAAaNQME4wHQYDVR0OBBYEFKzcbd6+N1TKfBjvmyTvw8+DnzAZ
MB8GA1UdIwQYMBaAFKzcbd6+N1TKfBjvmyTvw8+DnzAZMAwGA1UdEwQFMAMBAf8w
DQYJKoZIhvcNAQEFBQADggEBAAa1PpkpL842hh8jLXIpA/nK8aVDDcu5p3pA72/b
noFnZuKcuaSUOz1rrLqxDK2JB3lmChQaVx3pZwqJgA0h0XBScar+8wM2TfeyW+oU
Gr5tOAxoHVRpgn6oCoJkKo0HS2/NA12T/gYsXhXJXn4tuvDjaUzY+K+hhAWh64oL
/c61eKfCZKp50t9Eoua0xHII2Mveb27Ps46j/CZ1r0ts7sGieOqjQo3GZOOikG6F
vFY/2KV16/FdBovTFWMyKrzlYHm0Wgt28IWqhocq/golLfvkz3VAkLQvOF2i6hNc
4feBv69SRTsTCFN9PtJCtxPX/K9LZKeccBKgGjrHQpAF+JU=
-----END CERTIFICATE-----
"""
    cert1="""-----BEGIN CERTIFICATE-----
MIIEDzCCAvegAwIBAgIJAN9Ejmna3JJ8MA0GCSqGSIb3DQEBBQUAMIGdMQswCQYD
VQQGEwJSVTEVMBMGA1UECAwM0JzQvtGB0LrQstCwMTAwLgYDVQQKDCfQo9C00L7R
gdGC0L7QstC10YDRj9GO0YnQuNC5INGG0LXQvdGC0YAxIjAgBgNVBAMMGdCS0LjQ
utGC0L7RgCDQktCw0LPQvdC10YAxITAfBgkqhkiG9w0BCQEWEnZpdHVzQHdhZ25l
ci5wcC5ydTAeFw0xNDEwMjYxOTA3MTdaFw0yNDEwMjMxOTA3MTdaMIGBMQswCQYD
VQQGEwJSVTEVMBMGA1UECAwM0JzQvtGB0LrQstCwMRUwEwYDVQQHDAzQnNC+0YHQ
utCy0LAxIDAeBgNVBAoMF9Cn0LDRgdGC0L3QvtC1INC70LjRhtC+MSIwIAYDVQQD
DBnQktC40LrRgtC+0YAg0JLQsNCz0L3QtdGAMIIBIjANBgkqhkiG9w0BAQEFAAOC
AQ8AMIIBCgKCAQEArQSfrrxNROyzNEz60G2EHBP+E4BL0b1QytGAZZiQp2XIhhQe
b7mx+c4mpwgvD7/IdAcK+YVGx78nfY723T3wG48U7HzFNbLvNDycxyXecXbvCmRs
xPy8TxkwPf6TIT3UcixtwMMqZFqlAtSTDmOOWSaUuftL/+yFk729xDoYkOZhFwUS
UM5SbEZ0JpufWFjDi3Qwj3ZOTXliHC3e4C7187Me0Nne59dttyKpq1YAThn4Srar
vZYU6Ykk/LUae0FCvfeiKLShWY05XnPVmvPiiFTXJP8/Au8kfezlA4b+eS81zWq2
BFvNlBQsgf04S88oew0CuBBgtjUIIw7XZkS03QIDAQABo2wwajAJBgNVHRMEAjAA
MB0GA1UdDgQWBBRflZBerCFYheRQne/sWL3zY7GiAzAfBgNVHSMEGDAWgBSs3G3e
vjdUynwY75sk78PPg58wGTAdBgNVHREEFjAUgRJ2aXR1c0B3YWduZXIucHAucnUw
DQYJKoZIhvcNAQEFBQADggEBAGx1z0ylq90hP3x/2DmfVUYBA46CiGnV4NSiaOWE
Y18jCuG3W8FcI7JP4uEEjKyz3XbuhTFW2GsZ2L3FGgpA5eXBikgCn5kRpOHgb45r
SxE8u3TwVlYlaF+7RHPYLqmgb25d/O/28McemMmTGecPC9edbtDqLv03aJ0t4gXn
BD+xTJOP74Yhu5IPIV92J6pSBpIoy+qiyOA1iRpOWzrVHVR504vAaFxlfZs3VJhP
uo291iEXyooazJdbWwZwcwk7WrNNKhqktPTg0X1ZHNnGwOAGPzwNJFGPeFj71r0t
aFWU5EMRKaZK75keXq/RdaOAenl+nKF6xA2XHDhGgdndFfY=
-----END CERTIFICATE-----
"""
    pubkey1="""-----BEGIN PUBLIC KEY-----
MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEArQSfrrxNROyzNEz60G2E
HBP+E4BL0b1QytGAZZiQp2XIhhQeb7mx+c4mpwgvD7/IdAcK+YVGx78nfY723T3w
G48U7HzFNbLvNDycxyXecXbvCmRsxPy8TxkwPf6TIT3UcixtwMMqZFqlAtSTDmOO
WSaUuftL/+yFk729xDoYkOZhFwUSUM5SbEZ0JpufWFjDi3Qwj3ZOTXliHC3e4C71
87Me0Nne59dttyKpq1YAThn4SrarvZYU6Ykk/LUae0FCvfeiKLShWY05XnPVmvPi
iFTXJP8/Au8kfezlA4b+eS81zWq2BFvNlBQsgf04S88oew0CuBBgtjUIIw7XZkS0
3QIDAQAB
-----END PUBLIC KEY-----
"""
    digicert_cert="""digicert.crt
-----BEGIN CERTIFICATE-----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-----END CERTIFICATE-----
"""
    def test_readpubkey(self):
        c=X509(self.cert1)
        p=c.pubkey
        self.assertEqual(p.exportpub(),self.pubkey1)
    def test_pem(self):
        c=X509(self.cert1)
        self.assertEqual(c.pem(),self.cert1)
    def test_subject(self):
        c=X509(self.cert1)
        self.assertEqual(unicode(c.subject),u'C=RU,ST=Москва,L=Москва,O=Частное лицо,CN=Виктор Вагнер')
    def test_subject_str(self):
        c=X509(self.cert1)
        self.assertEqual(str(c.subject),b'C=RU,ST=\\D0\\9C\\D0\\BE\\D1\\81\\D0\\BA\\D0\\B2\\D0\\B0,L=\\D0\\9C\\D0\\BE\\D1\\81\\D0\\BA\\D0\\B2\\D0\\B0,O=\\D0\\A7\\D0\\B0\\D1\\81\\D1\\82\\D0\\BD\\D0\\BE\\D0\\B5 \\D0\\BB\\D0\\B8\\D1\\86\\D0\\BE,CN=\\D0\\92\\D0\\B8\\D0\\BA\\D1\\82\\D0\\BE\\D1\\80 \\D0\\92\\D0\\B0\\D0\\B3\\D0\\BD\\D0\\B5\\D1\\80')
    def test_subject_len(self):
        c=X509(self.cert1)
        self.assertEqual(len(c.subject),5)
    def test_issuer(self):
        c=X509(self.cert1)
        self.assertEqual(unicode(c.issuer),u'C=RU,ST=Москва,O=Удостоверяющий центр,CN=Виктор Вагнер,emailAddress=vitus@wagner.pp.ru')
    def test_subjectfields(self):
        c=X509(self.cert1)
        self.assertEqual(c.subject[Oid("C")],"RU")
        with self.assertRaises(TypeError):
            x=c.subject["CN"]
        self.assertEqual(c.subject[Oid("L")],u'\u041c\u043e\u0441\u043a\u0432\u0430')
    def test_subjectmodify(self):
        c=X509(self.cert1)
        with self.assertRaises(ValueError):
            c.subject[Oid("CN")]=u'Foo'
        with self.assertRaises(ValueError):
            del c.subject[Oid('CN')]
    def test_subjectbadsubfield(self):
        c=X509(self.cert1)
        with self.assertRaises(KeyError):
            x=c.subject[Oid("streetAddress")]
    def test_subjectfieldindex(self):
        c=X509(self.cert1)
        self.assertEqual(repr(c.subject[0]),repr((Oid('C'),u'RU')))
    def test_subjectbadindex(self):
        c=X509(self.cert1)
        with self.assertRaises(IndexError):
            x=c.subject[11]
        with self.assertRaises(IndexError):
            x=c.subject[-1]
    def test_notBefore(self):
        c=X509(self.cert1)
        self.assertEqual(c.startDate,datetime.datetime(2014,10,26,19,07,17,0,utc))
    def test_notAfter(self):
        c=X509(self.cert1)
        self.assertEqual(c.endDate,datetime.datetime(2024,10,23,19,7,17,0,utc))
    def test_subjectHash(self):
        c=X509(self.cert1)
        self.assertEqual(hash(c.subject),0x1f3ed722)
    def test_issuerHash(self):
        c=X509(self.cert1)
        self.assertEqual(hash(c.issuer),0x7d3ea8c3)
    def test_namecomp(self):
        c=X509(self.cert1)
        ca=X509(self.ca_cert)
        self.assertEqual(c.issuer,ca.subject)
        self.assertNotEqual(c.subject,c.issuer)
        self.assertEqual(ca.issuer,ca.subject)
    def test_serial(self):
        c=X509(self.cert1)
        self.assertEqual(c.serial,0xDF448E69DADC927CL)
    def test_version(self):
        c=X509(self.cert1)
        self.assertEqual(c.version,3)
    def test_ca_cert(self):
        ca=X509(self.ca_cert)
        self.assertTrue(ca.check_ca())
        notca=X509(self.cert1)
        self.assertFalse(notca.check_ca())
    def test_extension_count(self):
        cert=X509(self.cert1)
        self.assertTrue(len(cert.extensions),4)
        ca_cert=X509(self.ca_cert)
        self.assertEqual(len(ca_cert.extensions),3)
    def test_extension_outofrange(self):
        cert=X509(self.cert1)
        with self.assertRaises(IndexError):
            cert.extensions[4]
        with self.assertRaises(IndexError):
            cert.extensions[-1]
    def test_extension_oid(self):
        cert=X509(self.cert1)
        ext=cert.extensions[0]
        ext_id=ext.oid
        self.assertTrue(isinstance(ext_id,Oid))
        self.assertEqual(ext_id,Oid('basicConstraints'))
    def test_extension_text(self):
        cert=X509(self.cert1)
        ext=cert.extensions[0]
        self.assertEqual(str(ext),'CA:FALSE')
        self.assertEqual(unicode(ext),u'CA:FALSE')
    def test_extenson_find(self):
        cert=X509(self.cert1)
        exts=cert.extensions.find(Oid('subjectAltName'))
        self.assertEqual(len(exts),1)
        self.assertEqual(exts[0].oid,Oid('subjectAltName'))
    def test_extension_bad_find(self):
        cert=X509(self.cert1)
        with self.assertRaises(TypeError):
            exts=cert.extensions.find('subjectAltName')
    def test_extenson_critical(self):
        cert=X509(self.digicert_cert)
        crit_exts=cert.extensions.find_critical()
        self.assertEqual(len(crit_exts),2)
        other_exts=cert.extensions.find_critical(False)
        self.assertEqual(len(crit_exts)+len(other_exts),len(cert.extensions))
        self.assertEqual(crit_exts[0].critical,True)
        self.assertEqual(other_exts[0].critical,False)
    def test_verify_by_key(self):
        ca=X509(self.ca_cert)
        pubkey=ca.pubkey
        self.assertTrue(ca.verify(key=pubkey))
        c=X509(self.cert1)
        pk2=c.pubkey
        self.assertFalse(c.verify(key=pk2))
        self.assertTrue(c.verify(key=pubkey))
    def test_verify_self_singed(self):
        ca=X509(self.ca_cert)
        self.assertTrue(ca.verify())
    def test_default_filestore(self):
        store=X509Store(default=True)
        c1=X509(self.cert1)
        # Cert signed by our CA shouldn't be successfully verified
        # by default CA store
        self.assertFalse(c1.verify(store))
        # but cert, downloaded from some commercial CA - should.
        c2=X509(self.digicert_cert)
        self.assertTrue(c2.verify(store))
    def test_verify_by_filestore(self):
        trusted=NamedTemporaryFile(delete=False)
        trusted.write(self.ca_cert)
        trusted.close()
        goodcert=X509(self.cert1)
        badcert=X509(self.cert1[0:-30]+"GG"+self.cert1[-28:])
        gitcert=X509(self.digicert_cert)
        store=X509Store(file=trusted.name)
        os.unlink(trusted.name)
        # We should successfuly verify certificate signed by our CA cert
        self.assertTrue(goodcert.verify(store))
        # We should reject corrupted certificate
        self.assertFalse(badcert.verify(store))
        # And if we specify explicitely certificate file, certificate,
        # signed by some commercial CA should be rejected too
        self.assertFalse(gitcert.verify(store))
        trusted.close()
    def test_verify_by_dirstore(self):
        pass
    def test_certstack1(self):
        l=[]
        l.append(X509(self.cert1))
        self.assertEqual(unicode(l[0].subject[Oid('CN')]),u'Виктор Вагнер')
        l.append(X509(self.ca_cert))
        l.append(X509(self.digicert_cert))
        stack=StackOfX509(certs=l)
        self.assertEqual(len(stack),3)
        self.assertTrue(isinstance(stack[1],X509))
        self.assertEqual(unicode(stack[0].subject[Oid('CN')]),u'Виктор Вагнер')
        with self.assertRaises(IndexError):
            c=stack[-1]
        with self.assertRaises(IndexError):
            c=stack[3]
        del stack[1]
        self.assertEqual(len(stack),2)
        self.assertEqual(unicode(stack[0].subject[Oid('CN')]),u'Виктор Вагнер')
        self.assertEqual(unicode(stack[1].subject[Oid('CN')]),u'DigiCert High Assurance EV CA-1')
    def test_certstack2(self):
        stack=StackOfX509()
        stack.append(X509(self.cert1))
        stack.append(X509(self.ca_cert))
        c=stack[1]
        stack[1]=X509(self.digicert_cert)
        self.assertEqual(len(stack),2)
        self.assertEqual(unicode(stack[1].subject[Oid('CN')]),u'DigiCert High Assurance EV CA-1')
        with self.assertRaises(IndexError):
            stack[-1]=c
        with self.assertRaises(IndexError):
            stack[3]=c
        with self.assertRaises(TypeError):
            stack[0]=self.cert1
        with self.assertRaises(TypeError):
            stack.append(self.cert1)
    def test_certstack3(self):
        l=[]
        l.append(X509(self.cert1))
        self.assertEqual(unicode(l[0].subject[Oid('CN')]),u'Виктор Вагнер')
        l.append(X509(self.ca_cert))
        l.append(X509(self.digicert_cert))
        stack=StackOfX509(certs=l)
        stack2=StackOfX509(ptr=stack.ptr,disposable=False)
        with self.assertRaises(ValueError):
            stack3=StackOfX509(ptr=stack.ptr,certs=l)
        with self.assertRaises(ValueError):
            stack2[1]=l[0]
        with self.assertRaises(ValueError):
            stack2.append(l[0])
if __name__ == '__main__':
    unittest.main()



# -*- encoding: utf-8 -*-
"""
This module provides interface to OpenSSL MAC functions.

It has not only HMAC support, but can support other types of MAC.

"""

from ctypescrypto.digest import Digest,DigestType,DigestError
from ctypescrypto.oid import Oid
from ctypescrypto import libcrypto
from ctypes import c_int,c_char_p, c_void_p, c_size_t,POINTER,create_string_buffer,pointer

__all__ = ['MAC','DigestError']
class MAC(Digest):
    """
        This object represents MAC context. It is quite simular
        to digest algorithm. It is simular to hmac objects provided
        by standard library
    """
    def __init__(self,algorithm,key,digest=None,**kwargs):
        """
        Constructor has to obligatory arguments:
            
            @param algorithm - which is name of MAC algorithm i.e 'hmac' or 
                    'gost-mac' or equivalent Oid object
            @param key - byte buffer with key.

        Optional parameters are:
            digest - Oid or name of the digest algorithm to use. If none
                specified, OpenSSL will try to derive one from the MAC
                algorithm (or if algorithm is hmac, we'll substititute md5
                for compatibility with standard hmac module

            any other keyword argument is passed to EVP_PKEY_CTX as string
            option.

        """
        if isinstance(algorithm,str):
            self.algorithm=Oid(algorithm)
        elif isinstance(algorithm,Oid):
            self.algorithm=algorithm
        else:
            raise TypeError("Algorthm must be string or Oid")
        if self.algorithm==Oid('hmac') and digest is None:
                digest='md5'
        self.name=self.algorithm.shortname().lower()
        if digest is not None:
            self.digest_type=DigestType(digest)
            self.name+='-'+self.digest_type.digest_name
            d=self.digest_type.digest
        else:
            self.digest_type=None
            d=None
        self.key=libcrypto.EVP_PKEY_new_mac_key(self.algorithm.nid,None,key,len(key))
        if self.key is None:
            raise DigestError("EVP_PKEY_new_mac_key")
        pctx=c_void_p()
        self.ctx = self.newctx()
        if self.ctx == 0:
            raise DigestError("Unable to create digest context")
        if libcrypto.EVP_DigestSignInit(self.ctx,pointer(pctx),d,None,self.key) <= 0:
            raise DigestError("Unable to intialize digest context")
        self.digest_finalized=False
        if self.digest_type is None:
            self.digest_type=DigestType(Oid(libcrypto.EVP_MD_type(libcrypto.EVP_MD_CTX_md(self.ctx))))
        for (name,val) in kwargs.items():
            if libcrypto.EVP_PKEY_CTX_ctrl_str(pctx,name,val)<=0:
                raise DigestError("Unable to set mac parameter")
        self.digest_size = self.digest_type.digest_size
        self.block_size = self.digest_type.block_size
    def digest(self,data=None):
        """
        Method digest is redefined to return keyed MAC value instead of
        just digest.
        """
        if data is not None:
            self.update(data)
        b=create_string_buffer(256)
        size=c_size_t(256)
        if libcrypto.EVP_DigestSignFinal(self.ctx,b,pointer(size))<=0:
            raise DigestError('SignFinal')
        self.digest_finalized=True
        return b.raw[:size.value]

libcrypto.EVP_DigestSignFinal.argtypes=(c_void_p,c_char_p,POINTER(c_size_t))
libcrypto.EVP_DigestSignFinal.restype=c_int
libcrypto.EVP_DigestSignInit.argtypes=(c_void_p,POINTER(c_void_p),c_void_p,c_void_p,c_void_p)
libcrypto.EVP_DigestSignInit.restype=c_int
libcrypto.EVP_PKEY_CTX_ctrl_str.argtypes=(c_void_p,c_char_p,c_char_p)
libcrypto.EVP_PKEY_CTX_ctrl_str.restype=c_int
libcrypto.EVP_PKEY_new_mac_key.argtypes=(c_int,c_void_p,c_char_p,c_int)
libcrypto.EVP_PKEY_new_mac_key.restype=c_void_p
libcrypto.EVP_MD_CTX_md.argtypes=(c_void_p,)
libcrypto.EVP_MD_CTX_md.restype=c_void_p



"""
engine loading and configuration
"""
from ctypes import c_void_p, c_char_p, c_int
from ctypescrypto import libcrypto
from ctypescrypto.exception import LibCryptoError

__all__ = ['default', 'set_default', 'Engine']

default = None

class Engine(object):
    """
    Represents Openssl loadable module (engine).
    Allows to create PKey objects from private keys stored
    in the token, accessed by engine
    """
    def __init__(self, engine_id, **kwargs):
        eng = libcrypto.ENGINE_by_id(engine_id)
        if eng is None:
            # Try load engine
            eng = libcrypto.ENGINE_by_id("dynamic")
            if  eng is None:
                raise LibCryptoError("Cannot get 'dynamic' engine")
            if not libcrypto.ENGINE_ctrl_cmd_string(eng, "SO_PATH",
                                                    engine_id, 0):
                raise LibCryptoError("Cannot execute ctrl cmd SO_PATH")
            if not libcrypto.ENGINE_ctrl_cmd_string(eng, "LOAD", None, 0):
                raise LibCryptoError("Cannot execute ctrl cmd LOAD")
        if eng is None:
            raise ValueError("Cannot find engine " + engine)
        for cmd, value in kwargs.items():
            if not libcrypto.ENGINE_ctrl_cmd_string(eng, cmd, value, 0):
                raise LibCryptoError("Cannot execute ctrl cmd %s" % cmd)
        if not libcrypto.ENGINE_init(eng):
            raise LibCryptoError("Cannot initialize engine")
        self.ptr = eng

    def private_key(self, key_id, ui_method = None, ui_data=None):
        from ctypescrypto.pkey import PKey
        if ui_method is None:
            ui_ptr = libcrypto.UI_OpenSSL()
        else:
            ui_ptr = ui_method.ptr
        pkey = libcrypto.ENGINE_load_private_key(self.ptr, key_id, ui_ptr,
                                                 ui_data)
        if pkey is None:
            raise LibCryptoError("Cannot load private key")
        return PKey(ptr=pkey, cansign=True)

def set_default(eng, algorithms=0xFFFF):
    """
    Sets specified engine  as default for all
    algorithms, supported by it

    For compatibility with 0.2.x if string is passed instead
    of engine, attempts to load engine with this id
    """
    if not isinstance(eng,Engine):
        eng=Engine(eng)
    global default
    libcrypto.ENGINE_set_default(eng.ptr, c_int(algorithms))
    default = eng

# Declare function result and arguments for used functions
libcrypto.ENGINE_by_id.restype = c_void_p
libcrypto.ENGINE_by_id.argtypes = (c_char_p, )
libcrypto.ENGINE_set_default.argtypes = (c_void_p, c_int)
libcrypto.ENGINE_ctrl_cmd_string.argtypes = (c_void_p, c_char_p, c_char_p,
                                             c_int)
libcrypto.ENGINE_finish.argtypes = (c_char_p, )



"""
Exception which extracts libcrypto error information
"""
from ctypes import c_ulong, c_char_p, create_string_buffer
from ctypescrypto import libcrypto
strings_loaded = False

__all__ = ['LibCryptoError', 'clear_err_stack']

class LibCryptoError(Exception):
    """
    Exception for libcrypto errors. Adds all the info, which can be
    extracted from internal (per-thread) libcrypto error stack to the message,
    passed to the constructor.
    """
    def __init__(self, msg):
        global strings_loaded
        if not strings_loaded:
            libcrypto.ERR_load_crypto_strings()
            strings_loaded = True
        err_code = libcrypto.ERR_get_error()
        mesg = msg
        buf = create_string_buffer(128)
        while err_code != 0:
            mesg += "\n\t" + libcrypto.ERR_error_string(err_code, buf)
            err_code = libcrypto.ERR_get_error()
        super(LibCryptoError, self).__init__(mesg)

def clear_err_stack():
    """
    Clears internal libcrypto err stack. Call it if you've checked
    return code and processed exceptional situation, so subsequent
    raising of the LibCryptoError wouldn't list already handled errors
    """
    libcrypto.ERR_clear_error()

libcrypto.ERR_get_error.restype = c_ulong
libcrypto.ERR_error_string.restype = c_char_p
libcrypto.ERR_error_string.argtypes = (c_ulong, c_char_p)



# -*- encoding: utf-8 -*-
from ctypescrypto.oid import Oid
from base64 import b16decode,b16encode
from ctypescrypto.mac import *
from ctypescrypto.engine import set_default
import unittest

class TestMac(unittest.TestCase):
    def test_hmac_default(self):
        d=MAC('hmac',key='1234'*4)
        d.update('The Quick brown fox jumps over the lazy dog\n')
        self.assertEqual(d.name,'hmac-md5')
        self.assertEqual(d.hexdigest(),'A9C16D91CDF2A99273B72336D0D16B56')
    def test_hmac_digestdataa(self):
        d=MAC('hmac',key='1234'*4)
        h=d.hexdigest('The Quick brown fox jumps over the lazy dog\n')
        self.assertEqual(d.name,'hmac-md5')
        self.assertEqual(h,'A9C16D91CDF2A99273B72336D0D16B56')
    def test_hmac_byoid(self):
        d=MAC(Oid('hmac'),key='1234'*4)
        d.update('The Quick brown fox jumps over the lazy dog\n')
        self.assertEqual(d.name,'hmac-md5')
        self.assertEqual(d.hexdigest(),'A9C16D91CDF2A99273B72336D0D16B56')
    def test_mac_wrongtype(self):
        with self.assertRaises(TypeError):
            d=MAC(Oid('hmac').nid,key='1234'*4)
    def test_hmac_sha256(self):
        d=MAC('hmac',key='1234'*16,digest='sha256')
        d.update('The Quick brown fox jumps over the lazy dog\n')
        self.assertEqual(d.name,'hmac-sha256')
        self.assertEqual(d.hexdigest(),'BEBA086E1C67200664DCDEEC697D99DB1A8DAA72933A36B708FC5FD568173095')
    def test_gostmac(self):
        set_default('gost')
        d=MAC('gost-mac',key='1234'*8)
        d.update('The Quick brown fox jumps over the lazy dog\n')
        self.assertEqual(d.name,'gost-mac')
        self.assertEqual(d.digest_size,4)
        self.assertEqual(d.hexdigest(),'76F25AE3')
        with self.assertRaisesRegexp(DigestError,"invalid mac key length"):
            d=MAC('gost-mac',key='1234'*4)

if __name__ == "__main__":
    unittest.main()



from ctypescrypto.pkey import PKey
import unittest
from base64 import b64decode, b16decode

def pem2der(s):
    start=s.find('-----\n')
    finish=s.rfind('\n-----END')
    data=s[start+6:finish]
    return b64decode(data)

class TestPKey(unittest.TestCase):
    rsa="""-----BEGIN PRIVATE KEY-----
MIICdQIBADANBgkqhkiG9w0BAQEFAASCAl8wggJbAgEAAoGBAL9CzVZu9bczTmB8
776pPUoPo6WbAfwQqqiGrj91bk2mYE+MNLo4yIQH45IcwGzkyS8+YyQJf8Bux5BC
oZ2nwzXm5+JZkxkN1mtMzit2D7/hHmrZLoSbr0sxXFrD4a35RI4hXnSK9Sk01sXA
Te2OgHzm5nk1sG97G6SFq7CHe3gvAgMBAAECgYAgGV8K7Y5xk7vIt88oyZCOuHc3
mP9JRabOp+PgpJ3BjHXHg/lpc5Q7jHNmF0s4O2GEe0z6RFnbevwlOvmS0xAQ1hpg
5TnVVkiZvcJeQaZqWIlEOaLqA12YdhSyorfB6p3tfQ7ZmQusg3SCsru5kPJV4sm0
I+MuRCQZWSzIqelloQJBAPbtScJI0lXx8sktntZi69cAVvLtz5z1T7bZwFakNkNE
SUCjNc/hEEI6/1LScV8Kx9kgQ0+W8smu+GyUDceyVFECQQDGSeS7cTmojkiPQxPB
zb0vS+Nfpq6oYsx+gn5TBgMeWIZZrtMEaUU2o+rwsjKBP/gy6D1zC2b4W5O/A/7a
1GR/AkBUQhYoKKc1UpExGtMXfrvRKrmAvatZeM/Rqi4aooAtpfCFEOw82iStJOqY
/VxYPRqCuaKeVvjT31O/4SlumihxAkBahRU0NKYbuoiJThfQ23lIBB7SZadKG4A7
KJs+j3oQ+lyqyFJwqxX7sazpIJBJzMgjhT24LTZenn++LbbEcz1FAkBmDmxoq7qO
Ao6uTm8fnkD4C836wS4mYAPqwRBK1JvnEXEQee9irf+ip89BAg74ViTcGF9lwJwQ
gOM+X5Db+3pK
-----END PRIVATE KEY-----
"""
    rsakeytext="""Public-Key: (1024 bit)
Modulus:
    00:bf:42:cd:56:6e:f5:b7:33:4e:60:7c:ef:be:a9:
    3d:4a:0f:a3:a5:9b:01:fc:10:aa:a8:86:ae:3f:75:
    6e:4d:a6:60:4f:8c:34:ba:38:c8:84:07:e3:92:1c:
    c0:6c:e4:c9:2f:3e:63:24:09:7f:c0:6e:c7:90:42:
    a1:9d:a7:c3:35:e6:e7:e2:59:93:19:0d:d6:6b:4c:
    ce:2b:76:0f:bf:e1:1e:6a:d9:2e:84:9b:af:4b:31:
    5c:5a:c3:e1:ad:f9:44:8e:21:5e:74:8a:f5:29:34:
    d6:c5:c0:4d:ed:8e:80:7c:e6:e6:79:35:b0:6f:7b:
    1b:a4:85:ab:b0:87:7b:78:2f
Exponent: 65537 (0x10001)
"""
    ec1priv="""-----BEGIN PRIVATE KEY-----
MIGEAgEAMBAGByqGSM49AgEGBSuBBAAKBG0wawIBAQQgKnG6neqZvB98EEuuxnHs
fv+L/5abuNNG20wzUqRpncOhRANCAARWKXWeUZ6WiCKZ2kHx87jmJyx0G3ZB1iQC
+Gp2AJYswbQPhGPigKolzIbZYfwnn7QOca6N8QDhPAn3QQK8trZI
-----END PRIVATE KEY-----
"""
    ec1keytext="""Public-Key: (256 bit)
pub: 
    04:56:29:75:9e:51:9e:96:88:22:99:da:41:f1:f3:
    b8:e6:27:2c:74:1b:76:41:d6:24:02:f8:6a:76:00:
    96:2c:c1:b4:0f:84:63:e2:80:aa:25:cc:86:d9:61:
    fc:27:9f:b4:0e:71:ae:8d:f1:00:e1:3c:09:f7:41:
    02:bc:b6:b6:48
ASN1 OID: secp256k1
"""
    ec1pub="""-----BEGIN PUBLIC KEY-----
MFYwEAYHKoZIzj0CAQYFK4EEAAoDQgAEVil1nlGelogimdpB8fO45icsdBt2QdYk
AvhqdgCWLMG0D4Rj4oCqJcyG2WH8J5+0DnGujfEA4TwJ90ECvLa2SA==
-----END PUBLIC KEY-----
"""
    
    def test_unencrypted_pem(self):
        key=PKey(privkey=self.rsa)
        self.assertIsNotNone(key.key)
        self.assertEqual(str(key),self.rsakeytext)
    def test_export_priv_pem(self):
        key=PKey(privkey=self.ec1priv)
        out=key.exportpriv()
        self.assertEqual(self.ec1priv,out)
    def test_unencrypted_pem_ec(self):
        
        key=PKey(privkey=self.ec1priv)
        self.assertIsNotNone(key.key)
        self.assertEqual(str(key),self.ec1keytext)
    def test_unencrypted_der_ec(self):
        key=PKey(privkey=pem2der(self.ec1priv),format="DER")
        self.assertIsNotNone(key.key)
        self.assertEqual(str(key),self.ec1keytext)
    def test_pubkey_pem(self):
        key=PKey(pubkey=self.ec1pub)
        self.assertIsNotNone(key.key)   
        self.assertEqual(str(key),self.ec1keytext)
    def test_pubkey_der(self):
        key=PKey(pubkey=pem2der(self.ec1pub),format="DER")
        self.assertIsNotNone(key.key)   
        self.assertEqual(str(key),self.ec1keytext)
    def test_compare(self):
        key1=PKey(privkey=self.ec1priv)
        self.assertIsNotNone(key1.key)
        key2=PKey(pubkey=self.ec1pub)
        self.assertIsNotNone(key2.key)
        self.assertEqual(key1,key2)
    def test_sign(self):
        signer=PKey(privkey=self.ec1priv)
        digest=b16decode("FFCA2587CFD4846E4CB975B503C9EB940F94566AA394E8BD571458B9DA5097D5")
        signature=signer.sign(digest)
        self.assertTrue(len(signature)>0)
        verifier=PKey(pubkey=self.ec1pub)
        self.assertTrue(verifier.verify(digest,signature))
    def test_generate(self):
        newkey=PKey.generate("rsa")
        self.assertIsNotNone(newkey.key)
        s=str(newkey)
        self.assertEqual(s[:s.find("\n")],"Public-Key: (1024 bit)")
    def test_generate_params(self):
        newkey=PKey.generate("rsa",rsa_keygen_bits=2048)
        self.assertIsNotNone(newkey.key)
        s=str(newkey)
        self.assertEqual(s[:s.find("\n")],"Public-Key: (2048 bit)")
    def test_generate_ec(self):
        templkey=PKey(pubkey=self.ec1pub)
        newkey=PKey.generate("ec",paramsfrom=templkey)
        self.assertIsNotNone(newkey.key)
        s=str(newkey)
        self.assertEqual(s[:s.find("\n")],"Public-Key: (256 bit)")
        self.assertNotEqual(str(templkey),str(newkey))
if __name__ == "__main__":
    unittest.main()



"""
    Interface to some libcrypto functions

"""


from ctypes import CDLL, c_char_p
from ctypes.util import find_library
import sys

def config(filename=None):
    """
        Loads OpenSSL Config file. If none are specified, loads default
        (compiled in) one
    """
    libcrypto.OPENSSL_config(filename)

__all__ = ['config']

if sys.platform.startswith('win'):
    __libname__ = find_library('libeay32')
else:
    __libname__ = find_library('crypto')

if __libname__ is None:
    raise OSError("Cannot find OpenSSL crypto library")

#__libname__ = "/usr/local/ssl/lib/libcrypto.so.1.1"

libcrypto = CDLL(__libname__)
libcrypto.OPENSSL_config.argtypes = (c_char_p, )
libcrypto.OPENSSL_add_all_algorithms_conf()



"""
Implements interface to OpenSSL EVP_Digest* functions.

Interface  made as close to hashlib as possible.

This module is really an excess effort. Hashlib allows access to
mostly same functionality except oids and nids of hashing
algortithms (which might be needed for private key operations).

hashlib even allows to use engine-provided digests if it is build
with dinamically linked libcrypto - so use
ctypescrypto.engine.set_default("gost",xFFFF) and md_gost94
algorithm would be available both to this module and hashlib.

"""
from ctypes import c_int, c_char_p, c_void_p, POINTER, c_long, c_longlong
from ctypes import create_string_buffer, byref
from ctypescrypto import libcrypto
from ctypescrypto.exception import LibCryptoError
from ctypescrypto.oid import Oid
DIGEST_ALGORITHMS = ("MD5", "SHA1", "SHA224", "SHA256", "SHA384", "SHA512")

__all__ = ['DigestError', 'Digest', 'DigestType', 'new']

class DigestError(LibCryptoError):
    """ Exception raised if some OpenSSL function returns error """
    pass

def new(algname):
    """
    Behaves just like hashlib.new. Creates digest object by
    algorithm name
    """

    digest_type = DigestType(algname)
    return Digest(digest_type)

class DigestType(object):
    """
    Represents EVP_MD object - constant structure which describes
    digest algorithm
    """
    def __init__(self, digest_name):
        """
        Finds digest by its name. You can pass Oid object instead of
        name.

        Special case is when None is passed as name. In this case
        unitialized digest is created, and can be initalized later
        by setting its digest attribute to pointer to EVP_MD
        """
        if digest_name is None:
            return

        if isinstance(digest_name, Oid):
            self.digest_name = digest_name.longname()
        else:
            self.digest_name = str(digest_name)
        self.digest = libcrypto.EVP_get_digestbyname(self.digest_name)
        if self.digest is None:
            raise DigestError("Unknown digest: %s" % self.digest_name)

    @property
    def name(self):
        """ Returns name of the digest """
        if not hasattr(self, 'digest_name'):
            self.digest_name = Oid(libcrypto.EVP_MD_type(self.digest)
                                  ).longname()
        return self.digest_name

    def __del__(self):
        """ Empty destructor for constant object """
        pass

    @property
    def digest_size(self):
        """ Returns size of digest """
        return libcrypto.EVP_MD_size(self.digest)

    @property
    def block_size(self):
        """ Returns block size of the digest """
        return libcrypto.EVP_MD_block_size(self.digest)

    @property
    def oid(self):
        """ Returns Oid object of digest type """
        return Oid(libcrypto.EVP_MD_type(self.digest))

class Digest(object):
    """
    Represents EVP_MD_CTX object which actually used to calculate
    digests.
    """

    def __init__(self, digest_type):
        """
        Initializes digest using given type.
        """
        self.ctx = self.newctx()
        if self.ctx is None:
            raise DigestError("Unable to create digest context")
        self.digest_out = None
        self.digest_finalized = False
        result = libcrypto.EVP_DigestInit_ex(self.ctx, digest_type.digest, None)
        if result == 0:
            self._clean_ctx()
            raise DigestError("Unable to initialize digest")
        self.digest_type = digest_type
        self.digest_size = self.digest_type.digest_size
        self.block_size = self.digest_type.block_size

    def __del__(self):
        """ Uses _clean_ctx internal method """
        self._clean_ctx()

    def update(self, data, length=None):
        """
        Hashes given byte string

        @param data - string to hash
        @param length - if not specifed, entire string is hashed,
                otherwise only first length bytes
        """
        if self.digest_finalized:
            raise DigestError("No updates allowed")
        if not isinstance(data, str):
            raise TypeError("A string is expected")
        if length is None:
            length = len(data)
        elif length > len(data):
            raise ValueError("Specified length is greater than length of data")
        result = libcrypto.EVP_DigestUpdate(self.ctx, c_char_p(data), length)
        if result != 1:
            raise DigestError("Unable to update digest")

    def digest(self, data=None):
        """
        Finalizes digest operation and return digest value
        Optionally hashes more data before finalizing
        """
        if self.digest_finalized:
            return self.digest_out.raw[:self.digest_size]
        if data is not None:
            self.update(data)
        self.digest_out = create_string_buffer(256)
        length = c_long(0)
        result = libcrypto.EVP_DigestFinal_ex(self.ctx, self.digest_out,
                                              byref(length))
        if result != 1:
            raise DigestError("Unable to finalize digest")
        self.digest_finalized = True
        return self.digest_out.raw[:self.digest_size]
    def copy(self):
        """
        Creates copy of the digest CTX to allow to compute digest
        while being able to hash more data
        """

        new_digest = Digest(self.digest_type)
        libcrypto.EVP_MD_CTX_copy(new_digest.ctx, self.ctx)
        return new_digest

    def _clean_ctx(self):
        """
        Clears and deallocates context
        """
        try:
            if self.ctx is not None:
                libcrypto.EVP_MD_CTX_free(self.ctx)
                del self.ctx
        except AttributeError:
            pass
        self.digest_out = None
        self.digest_finalized = False

    def hexdigest(self, data=None):
        """
            Returns digest in the hexadecimal form. For compatibility
            with hashlib
        """
        from base64 import b16encode
        return b16encode(self.digest(data))


# Declare function result and argument types
libcrypto.EVP_get_digestbyname.restype = c_void_p
libcrypto.EVP_get_digestbyname.argtypes = (c_char_p, )
# These two functions are renamed in OpenSSL 1.1.0
if hasattr(libcrypto,"EVP_MD_CTX_create"):
    Digest.newctx = libcrypto.EVP_MD_CTX_create
    Digest.freectx = libcrypto.EVP_MD_CTX_destroy
else:
    Digest.newctx = libcrypto.EVP_MD_CTX_new
    Digest.freectx = libcrypto.EVP_MD_CTX_free
Digest.newctx.restype = c_void_p
Digest.freectx.argtypes = (c_void_p, )
# libcrypto.EVP_MD_CTX_create has no arguments
libcrypto.EVP_DigestInit_ex.restype = c_int
libcrypto.EVP_DigestInit_ex.argtypes = (c_void_p, c_void_p, c_void_p)
libcrypto.EVP_DigestUpdate.restype = c_int
libcrypto.EVP_DigestUpdate.argtypes = (c_void_p, c_char_p, c_longlong)
libcrypto.EVP_DigestFinal_ex.restype = c_int
libcrypto.EVP_DigestFinal_ex.argtypes = (c_void_p, c_char_p, POINTER(c_long))
libcrypto.EVP_MD_CTX_copy.restype = c_int
libcrypto.EVP_MD_CTX_copy.argtypes = (c_void_p, c_void_p)
libcrypto.EVP_MD_type.argtypes = (c_void_p, )
libcrypto.EVP_MD_size.argtypes = (c_void_p, )
libcrypto.EVP_MD_block_size.restype = c_int
libcrypto.EVP_MD_block_size.argtypes = (c_void_p, )
libcrypto.EVP_MD_size.restype = c_int
libcrypto.EVP_MD_size.argtypes = (c_void_p, )
libcrypto.EVP_MD_type.restype = c_int
libcrypto.EVP_MD_type.argtypes = (c_void_p, )



from ctypescrypto.bio import Membio
import unittest

class TestRead(unittest.TestCase):
    def test_readshortstr(self):
        s="A quick brown fox jumps over a lazy dog"
        bio=Membio(s)
        data=bio.read()
        self.assertEqual(data,s)
        data2=bio.read()
        self.assertEqual(data2,"")
        del bio
    def test_readwithlen(self):
        s="A quick brown fox jumps over a lazy dog"
        bio=Membio(s)
        data=bio.read(len(s))
        self.assertEqual(data,s)
        data2=bio.read(5)
        self.assertEqual(data2,"")
    def test_readwrongtype(self):
        s="A quick brown fox jumps over a lazy dog"
        bio=Membio(s)
        with self.assertRaises(TypeError):
            data=bio.read("5")
    def test_reset(self):
        s="A quick brown fox jumps over a lazy dog"
        bio=Membio(s)
        data=bio.read()
        bio.reset()
        data2=bio.read()
        del bio
        self.assertEqual(data,data2)
        self.assertEqual(data,s)
    def test_readlongstr(self):
        poem='''Eyes of grey--a sodden quay,
Driving rain and falling tears,
As the steamer wears to sea
In a parting storm of cheers.

Sing, for Faith and Hope are high--
None so true as you and I--
Sing the Lovers' Litany:
"Love like ours can never die!"

Eyes of black--a throbbing keel,
Milky foam to left and right;
Whispered converse near the wheel
In the brilliant tropic night.

Cross that rules the Southern Sky!
Stars that sweep and wheel and fly,
Hear the Lovers' Litany:
Love like ours can never die!"

Eyes of brown--a dusty plain
Split and parched with heat of June,
Flying hoof and tightened rein,
Hearts that beat the old, old tune.

Side by side the horses fly,
Frame we now the old reply
Of the Lovers' Litany:
"Love like ours can never die!"

Eyes of blue--the Simla Hills
Silvered with the moonlight hoar;
Pleading of the waltz that thrills,
Dies and echoes round Benmore.

"Mabel," "Officers," "Goodbye,"
Glamour, wine, and witchery--
On my soul's sincerity,
"Love like ours can never die!"

Maidens of your charity,
Pity my most luckless state.
Four times Cupid's debtor I--
Bankrupt in quadruplicate.

Yet, despite this evil case,
And a maiden showed me grace,
Four-and-forty times would I
Sing the Lovers' Litany:
"Love like ours can never die!"'''
        bio=Membio(poem)
        data=bio.read()
        self.assertEqual(data,poem)
        del bio
    def test_readparts(self):
        s="A quick brown fox jumps over the lazy dog"
        bio=Membio(s)
        a=bio.read(10)
        self.assertEqual(a,s[0:10])
        b=bio.read(9)
        self.assertEqual(b,s[10:19])
        c=bio.read()
        self.assertEqual(c,s[19:])
        d=bio.read()
        self.assertEqual(d,"")

class TestWrite(unittest.TestCase):
    def test_write(self):
        b=Membio()
        b.write("A quick brown ")
        b.write("fox jumps over ")
        b.write("the lazy dog.")
        self.assertEqual(str(b),"A quick brown fox jumps over the lazy dog.")

    def test_unicode(self):
        b=Membio()
        s='\xd0\xba\xd0\xb0\xd0\xba \xd1\x8d\xd1\x82\xd0\xbe \xd0\xbf\xd0\xbe-\xd1\x80\xd1\x83\xd1\x81\xd1\x81\xd0\xba\xd0\xb8'
        b.write(s)
        self.assertEqual(unicode(b),u'\u043a\u0430\u043a \u044d\u0442\u043e \u043f\u043e-\u0440\u0443\u0441\u0441\u043a\u0438')
    def test_unicode2(self):
        b=Membio()
        u=u'\u043a\u0430\u043a \u044d\u0442\u043e \u043f\u043e-\u0440\u0443\u0441\u0441\u043a\u0438'
        b.write(u)
        self.assertEqual(unicode(b),u)
if __name__ == '__main__':
    unittest.main()



from ctypescrypto.oid import Oid
from ctypescrypto.ec import create
from base64 import b16decode
import unittest



class TestEcCreation(unittest.TestCase):
    ec1priv="""-----BEGIN PRIVATE KEY-----
MIGEAgEAMBAGByqGSM49AgEGBSuBBAAKBG0wawIBAQQgKnG6neqZvB98EEuuxnHs
fv+L/5abuNNG20wzUqRpncOhRANCAARWKXWeUZ6WiCKZ2kHx87jmJyx0G3ZB1iQC
+Gp2AJYswbQPhGPigKolzIbZYfwnn7QOca6N8QDhPAn3QQK8trZI
-----END PRIVATE KEY-----
"""
    bigkey="""-----BEGIN PRIVATE KEY-----
MIGEAgEAMBAGByqGSM49AgEGBSuBBAAKBG0wawIBAQQgAAAAAAAAAAAAAAAAAAAA
AUVRIxlQt1/EQC2hcy/Jvr6hRANCAASRZsKJufkF5V+ePfn2nX81a0oiCV+JT0cV
cUqktWYGr/GB65Zr5Ky1z/nha2bYCb6U4hTwbJP9CRCZr5hJklXn
-----END PRIVATE KEY-----
"""
    def test_keyone(self):
        key=create(Oid("secp256k1"),b16decode("2A71BA9DEA99BC1F7C104BAEC671EC7EFF8BFF969BB8D346DB4C3352A4699DC3",True))
            
        out=key.exportpriv()
        self.assertEqual(out,self.ec1priv)

    def test_bignum(self):
        keyval='\xff'*32
        key=create(Oid("secp256k1"),keyval)
        self.assertEqual(key.exportpriv(),self.bigkey)
if __name__ == "__main__":
    unittest.main()



"""
access to symmetric ciphers from libcrypto

"""
from ctypes import create_string_buffer, c_char_p, c_void_p, c_int
from ctypes import byref, POINTER
from ctypescrypto import libcrypto
from ctypescrypto.exception import LibCryptoError
from ctypescrypto.oid import Oid

CIPHER_ALGORITHMS = ("DES", "DES-EDE3", "BF", "AES-128", "AES-192", "AES-256")
CIPHER_MODES = ("STREAM", "ECB", "CBC", "CFB", "OFB", "CTR", "GCM")

#

__all__ = ['CipherError', 'new', 'Cipher', 'CipherType']

class CipherError(LibCryptoError):
    """
    Exception raise when OpenSSL function returns error
    """
    pass

def new(algname, key, encrypt=True, iv=None):
    """
    Returns new cipher object ready to encrypt-decrypt data

    @param algname - string algorithm name like in opemssl command
                     line
    @param key - binary string representing ciher key
    @param encrypt - if True (default) cipher would be initialized
                    for encryption, otherwise - for decrypton
    @param iv - initialization vector
    """
    ciph_type = CipherType(algname)
    return Cipher(ciph_type, key, iv, encrypt)

class CipherType(object):
    """
    Describes cihper algorihm. Can be used to produce cipher
    instance and to get various information about cihper
    """

    def __init__(self, cipher_name):
        """
        Constructs cipher algortihm using textual name as in openssl
        command line
        """
        self.cipher = libcrypto.EVP_get_cipherbyname(cipher_name)
        if self.cipher is None:
            raise CipherError("Unknown cipher: %s" % cipher_name)

    def __del__(self):
        """
        It is constant object with do-nothing del
        """
        pass

    def block_size(self):
        """
        Returns block size of the cipher
        """
        return libcrypto.EVP_CIPHER_block_size(self.cipher)

    def key_length(self):
        """
        Returns key length of the cipher
        """
        return libcrypto.EVP_CIPHER_key_length(self.cipher)

    def iv_length(self):
        """
        Returns initialization vector length of the cipher
        """
        return libcrypto.EVP_CIPHER_iv_length(self.cipher)

    def flags(self):
        """
        Return cipher flags. Low three bits of the flags encode
        cipher mode (see mode). Higher bits  is combinatuon of
        EVP_CIPH* constants defined in the <openssl/evp.h>
        """
        return libcrypto.EVP_CIPHER_flags(self.cipher)

    def mode(self):
        """
        Returns cipher mode as string constant like CBC, OFB etc.
        """
        return CIPHER_MODES[self.flags() & 0x7]

    def algo(self):
        """
        Return cipher's algorithm name, derived from OID
        """
        return self.oid().shortname()

    def oid(self):
        """
        Returns ASN.1 object identifier of the cipher as
        ctypescrypto.oid.Oid object
        """
        return Oid(libcrypto.EVP_CIPHER_nid(self.cipher))

class Cipher(object):
    """
    Performs actual encrypton decryption
    Note that object keeps some internal state.
    To obtain full ciphertext (or plaintext during decihpering)
    user should concatenate results of all calls of update with
    result of finish
    """
    def __init__(self, cipher_type, key, iv, encrypt=True):
        """
        Initializing cipher instance.

        @param cipher_type - CipherType object
        @param key = binary string representing the key
        @param iv - binary string representing initializtion vector
        @param encrypt - if True(default) we ere encrypting.
               Otherwise decrypting

        """
        self._clean_ctx()
        # Check key and iv length
        if key is None:
            raise ValueError("No key specified")

        key_ptr = c_char_p(key)
        iv_ptr = c_char_p(iv)
        self.ctx = libcrypto.EVP_CIPHER_CTX_new()
        if self.ctx == 0:
            raise CipherError("Unable to create cipher context")
        self.encrypt = encrypt
        enc = 1 if encrypt else 0
        if not iv is None and len(iv) != cipher_type.iv_length():
            raise ValueError("Invalid IV length for this algorithm")

        if len(key) != cipher_type.key_length():
            if (cipher_type.flags() & 8) != 0:
                # Variable key length cipher.
                result = libcrypto.EVP_CipherInit_ex(self.ctx,
                                                     cipher_type.cipher,
                                                     None, None, None,
                                                     c_int(enc))
                result = libcrypto.EVP_CIPHER_CTX_set_key_length(self.ctx,
                                                                 len(key))
                if result == 0:
                    self._clean_ctx()
                    raise CipherError("Unable to set key length")
                result = libcrypto.EVP_CipherInit_ex(self.ctx, None, None,
                                                     key_ptr, iv_ptr,
                                                     c_int(enc))
            else:
                raise ValueError("Invalid key length for this algorithm")
        else:
            result = libcrypto.EVP_CipherInit_ex(self.ctx, cipher_type.cipher,
                                                 None, key_ptr, iv_ptr,
                                                 c_int(enc))
        if result == 0:
            self._clean_ctx()
            raise CipherError("Unable to initialize cipher")
        self.cipher_type = cipher_type
        self.block_size = self.cipher_type.block_size()
        self.cipher_finalized = False

    def __del__(self):
        """
        We define _clean_ctx() to do all the cleanup
        """
        self._clean_ctx()

    def padding(self, padding=True):
        """
        Sets padding mode of the cipher
        """
        padding_flag = 1 if padding else 0
        libcrypto.EVP_CIPHER_CTX_set_padding(self.ctx, padding_flag)

    def update(self, data):
        """
        Performs actual encrypton/decrypion

        @param data - part of the plain text/ciphertext to process
        @returns - part of ciphercext/plain text

        Passed chunk of text doesn't need to contain full ciher
        blocks. If neccessery, part of passed data would be kept
        internally until next data would be received or finish
        called
        """
        if self.cipher_finalized:
            raise CipherError("No updates allowed")
        if not isinstance(data, str):
            raise TypeError("A string is expected")
        if len(data) == 0:
            return ""
        outbuf = create_string_buffer(self.block_size+len(data))
        outlen = c_int(0)
        ret = libcrypto.EVP_CipherUpdate(self.ctx, outbuf, byref(outlen),
                                         data, len(data))
        if ret <= 0:
            self._clean_ctx()
            self.cipher_finalized = True
            raise CipherError("problem processing data")
        return outbuf.raw[:int(outlen.value)]

    def finish(self):
        """
        Finalizes processing. If some data are kept in the internal
        state, they would be processed and returned.
        """
        if self.cipher_finalized:
            raise CipherError("Cipher operation is already completed")
        outbuf = create_string_buffer(self.block_size)
        self.cipher_finalized = True
        outlen = c_int(0)
        result = libcrypto.EVP_CipherFinal_ex(self.ctx, outbuf, byref(outlen))
        if result == 0:
            self._clean_ctx()
            raise CipherError("Unable to finalize cipher")
        if outlen.value > 0:
            return outbuf.raw[:int(outlen.value)]
        else:
            return ""

    def _clean_ctx(self):
        """
        Cleans up cipher ctx and deallocates it
        """
        try:
            if self.ctx is not None:
                self.__ctxcleanup(self.ctx)
                libcrypto.EVP_CIPHER_CTX_free(self.ctx)
                del self.ctx
        except AttributeError:
            pass
        self.cipher_finalized = True


#
# Used C function block_size
#
libcrypto.EVP_CIPHER_block_size.argtypes = (c_void_p, )

#Function EVP_CIPHER_CTX_cleanup renamed to EVP_CIPHER_CTX_reset
# in the OpenSSL 1.1.0
if hasattr(libcrypto,"EVP_CIPHER_CTX_cleanup"):
    Cipher.__ctxcleanup = libcrypto.EVP_CIPHER_CTX_cleanup 
else:
    Cipher.__ctxcleanup = libcrypto.EVP_CIPHER_CTX_reset
Cipher.__ctxcleanup.argtypes  = (c_void_p, )
libcrypto.EVP_CIPHER_CTX_free.argtypes = (c_void_p, )
libcrypto.EVP_CIPHER_CTX_new.restype = c_void_p
libcrypto.EVP_CIPHER_CTX_set_padding.argtypes = (c_void_p, c_int)
libcrypto.EVP_CipherFinal_ex.argtypes = (c_void_p, c_char_p, POINTER(c_int))
libcrypto.EVP_CIPHER_flags.argtypes = (c_void_p, )
libcrypto.EVP_CipherInit_ex.argtypes = (c_void_p, c_void_p, c_void_p, c_char_p,
                                        c_char_p, c_int)
libcrypto.EVP_CIPHER_iv_length.argtypes = (c_void_p, )
libcrypto.EVP_CIPHER_key_length.argtypes = (c_void_p, )
libcrypto.EVP_CIPHER_nid.argtypes = (c_void_p, )
libcrypto.EVP_CipherUpdate.argtypes = (c_void_p, c_char_p, POINTER(c_int),
                                       c_char_p, c_int)
libcrypto.EVP_get_cipherbyname.restype = c_void_p
libcrypto.EVP_get_cipherbyname.argtypes = (c_char_p, )
libcrypto.EVP_CIPHER_CTX_set_key_length.argtypes = (c_void_p, c_int)



"""
Interface to OpenSSL object identifier database.

It is primarily intended to deal with OIDs which are compiled into the
database or defined in the openssl configuration files.

But see create() function.

OpenSSL maintains database of OIDs, which contain long and short
human-readable names, which correspond to Oid as well as canonical
dotted-decimal representation, and links it to small integer, named
numeric identifier or 'nid'. Most OpenSSL functions which deals with
ASN.1 structures such as certificates or cryptographic messages,
expect or return nids, but it is very bad idea to hardcode nids into
your app, because it can change after mere recompilation of OpenSSL
library.

This module provides Oid object which represents entry to OpenSSL
OID database.
"""
from ctypescrypto import libcrypto
from ctypes import c_char_p, c_void_p, c_int, create_string_buffer
from ctypescrypto.exception import LibCryptoError

__all__ = ['Oid', 'create', 'cleanup']

class Oid(object):
    """
    Represents an OID (ASN.1 Object identifier).


    It can be consturucted by textual
    representation like Oid("commonName") or Oid("CN"),
    dotted-decimal Oid("1.2.3.4") or using OpenSSL numeric
    identifer (NID), which is typically returned or required by
    OpenSSL API functions. If object is consturcted from textual
    representation which is not present in the database, it fails
    with ValueError

    attribute nid - contains object nid.
    """

    def __init__(self, value):
        """
        Object constuctor. Accepts string, integer, or another Oid
        object.

        Integer should be OpenSSL numeric identifier (nid) as returned
        by some libcrypto function or extracted from some libcrypto
        structure
        """
        if isinstance(value, unicode):
            value = value.encode('ascii')
        if isinstance(value, str):
            self.nid = libcrypto.OBJ_txt2nid(value)
            if self.nid == 0:
                raise ValueError("Cannot find object %s in the database" %
                                 value)
        elif isinstance(value, (int, long)):
            short = libcrypto.OBJ_nid2sn(value)
            if short is None:
                raise ValueError("No such nid %d in the database" % value)
            self.nid = value
        elif isinstance(value, Oid):
            self.nid = value.nid
        else:
            raise TypeError("Cannot convert this type to object identifier")
    def __hash__(self):
        " Hash of object is equal to nid because Oids with same nid are same"
        return self.nid
    def __cmp__(self, other):
        " Compares NIDs of two objects "
        return self.nid - other.nid
    def __str__(self):
        " Default string representation of Oid is dotted-decimal "
        return self.dotted()
    def __repr__(self):
        " Returns constructor call of Oid with dotted representation "
        return "Oid('%s')" % (self.dotted())
    def shortname(self):
        " Returns short name if any "
        return libcrypto.OBJ_nid2sn(self.nid)
    def longname(self):
        " Returns logn name if any "
        return  libcrypto.OBJ_nid2ln(self.nid)
    def dotted(self):
        " Returns dotted-decimal reperesentation "
        obj = libcrypto.OBJ_nid2obj(self.nid)
        buf = create_string_buffer(256)
        libcrypto.OBJ_obj2txt(buf, 256, obj, 1)
        return buf.value
    @staticmethod
    def fromobj(obj):
        """
        Creates an OID object from the pointer to ASN1_OBJECT c structure.
        This method intended for internal use for submodules which deal
        with libcrypto ASN1 parsing functions, such as x509 or CMS
        """
        nid = libcrypto.OBJ_obj2nid(obj)
        if nid == 0:
            buf = create_string_buffer(80)
            dotted_len = libcrypto.OBJ_obj2txt(buf, 80, obj, 1)
            dotted = buf[:dotted_len]
            oid = create(dotted, dotted, dotted)
        else:
            oid = Oid(nid)
        return oid

def create(dotted, shortname, longname):
    """
    Creates new OID in the database

    @param dotted - dotted-decimal representation of new OID
    @param shortname - short name for new OID
    @param longname - long name for new OID

    @returns Oid object corresponding to new OID

    This function should be used with exreme care. Whenever
    possible, it is better to add new OIDs via OpenSSL configuration
    file

    Results of calling this function twice for same OIDor for
    Oid alredy in database are undefined

    """
    nid = libcrypto.OBJ_create(dotted, shortname, longname)
    if nid == 0:
        raise LibCryptoError("Problem adding new OID to the  database")
    return Oid(nid)

def cleanup():
    """
    Removes all the objects, dynamically added by current
    application from database.
    """
    libcrypto.OBJ_cleanup()

libcrypto.OBJ_nid2sn.restype = c_char_p
libcrypto.OBJ_nid2ln.restype = c_char_p
libcrypto.OBJ_nid2obj.restype = c_void_p
libcrypto.OBJ_obj2txt.argtypes = (c_char_p, c_int, c_void_p, c_int)
libcrypto.OBJ_txt2nid.argtupes = (c_char_p, )
libcrypto.OBJ_create.argtypes = (c_char_p, c_char_p, c_char_p)



from ctypescrypto.oid import Oid
from ctypescrypto import digest
from base64 import b16decode,b16encode
import unittest

class TestDigestType(unittest.TestCase):
    def test_md4(self):
        d=digest.DigestType("md4")
        self.assertEqual(d.digest_size,16)
        self.assertEqual(d.block_size,64)
        self.assertEqual(d.oid,Oid("md4"))
        self.assertEqual(d.name,'md4')
    def test_md5(self):
        d=digest.DigestType("md5")
        self.assertEqual(d.digest_size,16)
        self.assertEqual(d.block_size,64)
        self.assertEqual(d.oid,Oid("md5"))
        self.assertEqual(d.name,'md5')
    def test_sha1(self):
        d=digest.DigestType("sha1")
        self.assertEqual(d.digest_size,20)
        self.assertEqual(d.block_size,64)
        self.assertEqual(d.oid,Oid("sha1"))
        self.assertEqual(d.name,'sha1')
    def test_sha256(self):
        d=digest.DigestType("sha256")
        self.assertEqual(d.digest_size,32)
        self.assertEqual(d.block_size,64)
        self.assertEqual(d.oid,Oid("sha256"))
        self.assertEqual(d.name,'sha256')
    def test_sha384(self):
        d=digest.DigestType("sha384")
        self.assertEqual(d.digest_size,48)
        self.assertEqual(d.block_size,128)
        self.assertEqual(d.oid,Oid("sha384"))
        self.assertEqual(d.name,'sha384')
    def test_sha512(self):
        d=digest.DigestType("sha512")
        self.assertEqual(d.digest_size,64)
        self.assertEqual(d.block_size,128)
        self.assertEqual(d.oid,Oid("sha512"))
        self.assertEqual(d.name,'sha512')
    def test_createfromoid(self):
        oid=Oid('sha256')
        d=digest.DigestType(oid)
        self.assertEqual(d.digest_size,32)
        self.assertEqual(d.block_size,64)
        self.assertEqual(d.oid,Oid("sha256"))
        self.assertEqual(d.name,'sha256')
    def test_createfromEVP_MD(self):
        d1=digest.DigestType("sha256")
        d2=digest.DigestType(None)
        with self.assertRaises(AttributeError):
            s=d2.name
        d2.digest=d1.digest
        self.assertEqual(d2.digest_size,32)
        self.assertEqual(d2.block_size,64)
        self.assertEqual(d2.oid,Oid("sha256"))
        self.assertEqual(d2.name,'sha256')
    def test_invalidDigest(self):
        with self.assertRaises(digest.DigestError):
            d=digest.DigestType("no-such-digest")


class TestIface(unittest.TestCase):
    """ Test all methods with one algorithms """
    msg="A quick brown fox jumps over the lazy dog."
    dgst="00CFFE7312BF9CA73584F24BDF7DF1D028340397"
    def test_cons(self):
        md=digest.DigestType("sha1")
        dgst=digest.Digest(md)
        dgst.update(self.msg)
        self.assertEqual(dgst.digest_size,20)
        self.assertEqual(dgst.hexdigest(),self.dgst)
    def test_digestwithdata(self):
        md=digest.DigestType("sha1")
        dgst=digest.Digest(md)
        self.assertEqual(dgst.digest(self.msg),b16decode(self.dgst))
    def test_length(self):
        l=len(self.msg)
        msg=self.msg+" Dog barks furiously."
        dgst=digest.new("sha1")
        dgst.update(msg,length=l)
        self.assertEqual(dgst.hexdigest(),self.dgst)
    def test_badlength(self):
        l=len(self.msg)
        dgst=digest.new("sha1")
        with self.assertRaises(ValueError):
            dgst.update(self.msg,length=l+1)
    def test_bindigest(self):
        dgst=digest.new("sha1")
        dgst.update(self.msg)
        self.assertEqual(dgst.digest_size,20)
        self.assertEqual(dgst.digest(),b16decode(self.dgst,True)) 
    def test_duplicatedigest(self):
        dgst=digest.new("sha1")
        dgst.update(self.msg)
        v1=dgst.digest()
        v2=dgst.digest()
        self.assertEqual(v1,v2)
    def test_updatefinalized(self):
        dgst=digest.new("sha1")
        dgst.update(self.msg)
        h=dgst.hexdigest()
        with self.assertRaises(digest.DigestError):
            dgst.update(self.msg)
    def test_wrongtype(self):
        dgst=digest.new("sha1")
        with self.assertRaises(TypeError):
            dgst.update(['a','b','c'])
        with self.assertRaises(TypeError):
            dgst.update(18)
        with self.assertRaises(TypeError):
            dgst.update({"a":"b","c":"D"})
        with self.assertRaises(TypeError):
            dgst.update(u'\u0430\u0431')
    def test_copy(self):
        dgst=digest.new("sha1")
        dgst.update("A quick brown fox jumps over ")
        d2=dgst.copy()
        dgst.update("the lazy dog.")
        value1=dgst.hexdigest()
        d2.update("the fat pig.")
        value2=d2.hexdigest()
        self.assertEqual(value1,"00CFFE7312BF9CA73584F24BDF7DF1D028340397")
        self.assertEqual(value2,"5328F33739BEC2A15B6A30F17D3BC13CC11A7C78")
class TestAlgo(unittest.TestCase):
    """ Test all statdard algorithms """
    def test_md5(self):
        d=digest.new("md5")
        self.assertEqual(d.digest_size,16)
        d.update("A quick brown fox jumps over the lazy dog.")
        self.assertEqual(d.hexdigest(),"DF756A3769FCAB0A261880957590C768")

    def test_md4(self):
        d=digest.new("md4")
        d.update("A quick brown fox jumps over the lazy dog.")
        self.assertEqual(d.digest_size,16)
        self.assertEqual(d.hexdigest(),"FAAED595A3E38BBF0D9B4B98021D200F")
    def test_sha256(self):
        d=digest.new("sha256")
        d.update("A quick brown fox jumps over the lazy dog.")
        self.assertEqual(d.digest_size,32)
        self.assertEqual(d.hexdigest(),"FFCA2587CFD4846E4CB975B503C9EB940F94566AA394E8BD571458B9DA5097D5")
    def test_sha384(self):
        d=digest.new("sha384")
        d.update("A quick brown fox jumps over the lazy dog.")
        self.assertEqual(d.digest_size,48)
        self.assertEqual(d.hexdigest(),"C7D71B1BA81D0DD028E79C7E75CF2F83169C14BA732CA5A2AD731151584E9DE843C1A314077D62B96B03367F72E126D8")
    def test_sha512(self):
        d=digest.new("sha512")
        self.assertEqual(d.digest_size,64)
        d.update("A quick brown fox jumps over the lazy dog.")
        self.assertEqual(d.hexdigest(),"3045575CF3B873DD656F5F3426E04A4ACD11950BB2538772EE14867002B408E21FF18EF7F7B2CAB484A3C1C0BE3F8ACC4AED536A427353C7748DC365FC1A8646")
    def test_wrongdigest(self):
        with self.assertRaises(digest.DigestError):
            dgst=digest.new("no-such-digest")

if __name__ == "__main__":
    unittest.main()



from ctypescrypto.pbkdf2 import pbkdf2
import unittest

class TestPBKDF2(unittest.TestCase):
    answersha1='\xc13\xb3\xc8\x80\xc2\t\x01\xdaR]\x08\x03\xaf>\x85\xed\x9bU\xf0\x89\n\x81Ctu\xee\xe3\xfe\xd9\xfd\x85\xe2"\x8c\xfbQ\xfeb4\x8f(ZF\xfd\xc3w\x13'
    answersha256='oY\xaf\xf7\xfeB7@\xa80%\t\'\xd5r0\xbe\xb4\xf7\xe6TQ\xd2|Tx\xc0e\xff[0a\xe56\xec\xff\xda\xcd\xed~\xbde\xad"\xe8\t\x01o'
    answersha1_1000='\xe9\xfe\xbf\xf5K\xfc\xe6h\xfd\xe3\x01\xac\xc8Uc\xcc\x9d\xc7\x1e\xf6\xf8\xd7\xaa\xef\x06se\xbe\x0e^e"\xefa\xba\xe1\xb0\x0b\xc1;\xcd\x05G<\xcc\rE\xfb'
    def test_defaults(self):
        d=pbkdf2("password","saltsalt",48)
        self.assertEqual(d,self.answersha1)
    def test_sha1(self):
        d=pbkdf2("password","saltsalt",48,digesttype="sha1",iterations=2000)
        self.assertEqual(d,self.answersha1)
    def test_1000iter(self):
        d=pbkdf2("password","saltsalt",48,digesttype="sha1",iterations=1000)
        self.assertEqual(d,self.answersha1_1000)
    def test_sha256(self):  
        d=pbkdf2("password","\01\02\03\04\0abc",48,digesttype="sha256")
        self.assertEqual(d,self.answersha256)
        
if __name__ == "__main__":
    unittest.main()



from distutils.core import setup
import distutils.cmd
import sys, os

class MyTests(distutils.cmd.Command):
    user_options = []
    def initialize_options(self):
        pass
    def finalize_options(self):
        pass
    def run(self):
        sys.path.insert(0, os.getcwd())
        import unittest
        result = unittest.TextTestResult(sys.stdout, True, True)
        suite = unittest.defaultTestLoader.discover("./tests")
        print "Discovered %d test cases" % suite.countTestCases()
        result.buffer = True
        suite.run(result)
        print ""
        if not result.wasSuccessful():
            if len(result.errors):
                print "============ Errors disovered ================="
                for res in result.errors:
                    print res[0], ":", res[1]

            if len(result.failures):
                print "============ Failures disovered ================="
                for res in result.failures:
                    print res[0], ":", res[1]
            sys.exit(1)
        else:
            print "All tests successful"

setup(
    name="ctypescrypto",
    version="0.4.0",
    description="CTypes-based interface for some OpenSSL libcrypto features",
    author="Victor Wagner",
    author_email="vitus@wagner.pp.ru",
    url="https://github.com/vbwagner/ctypescrypto",
    packages=["ctypescrypto"],
    cmdclass={"test":MyTests}
)




"""
Implements operations with CMS EnvelopedData and SignedData messages

Contains function CMS() which parses CMS message and creates either
EnvelopedData or SignedData objects (EncryptedData and CompressedData
can be easily added, because OpenSSL contain nessesary function)

Each of these objects contains create() static method which is used to
create it from raw data and neccessary certificates.


"""
from ctypes import c_int, c_void_p, c_char_p, c_int, c_uint, c_size_t, POINTER
from ctypescrypto.exception import LibCryptoError
from ctypescrypto import libcrypto
from ctypescrypto.bio import Membio
from ctypescrypto.oid import Oid
from ctypescrypto.x509 import StackOfX509

class CMSError(LibCryptoError):
    """
    Exception which is raised when error occurs
    """
    pass

class Flags:
    """
    Constants for flags passed to the CMS methods.
    Can be OR-ed together
    """
    TEXT = 1
    NOCERTS = 2
    NO_CONTENT_VERIFY = 4
    NO_ATTR_VERIFY = 8
    NO_SIGS = NO_CONTENT_VERIFY | NO_ATTR_VERIFY
    NOINTERN = 0x10
    NO_SIGNER_CERT_VERIFY = 0x20
    NO_VERIFY = 0x20
    DETACHED = 0x40
    BINARY = 0x80
    NOATTR = 0x100
    NOSMIMECAP = 0x200
    NOOLDMIMETYPE = 0x400
    CRLFEOL = 0x800
    STREAM = 0x1000
    NOCRL = 0x2000
    PARTIAL = 0x4000
    REUSE_DIGEST = 0x8000
    USE_KEYID = 0x10000
    DEBUG_DECRYPT = 0x20000

def CMS(data, format="PEM"):
    """
    Parses CMS data and returns either SignedData or EnvelopedData
    object
    """
    bio = Membio(data)
    if format == "PEM":
        ptr = libcrypto.PEM_read_bio_CMS(bio.bio, None, None, None)
    else:
        ptr = libcrypto.d2i_CMS_bio(bio.bio, None)
    typeoid = Oid(libcrypto.OBJ_obj2nid(libcrypto.CMS_get0_type(ptr)))
    if typeoid.shortname() == "pkcs7-signedData":
        return SignedData(ptr)
    elif typeoid.shortname() == "pkcs7-envelopedData":
        return EnvelopedData(ptr)
    elif typeoid.shortname() == "pkcs7-encryptedData":
        return EncryptedData(ptr)
    else:
        raise NotImplementedError("cannot handle "+typeoid.shortname())

class CMSBase(object):
    """
    Common ancessor for all CMS types.
    Implements serializatio/deserialization
    """
    def __init__(self, ptr=None):
        self.ptr = ptr
    def __str__(self):
        """
        Serialize in DER format
        """
        bio = Membio()
        if not libcrypto.i2d_CMS_bio(bio.bio, self.ptr):
            raise CMSError("writing CMS to PEM")
        return str(bio)

    def pem(self):
        """
        Serialize in PEM format
        """
        bio = Membio()
        if not libcrypto.PEM_write_bio_CMS(bio.bio, self.ptr):
            raise CMSError("writing CMS to PEM")
        return str(bio)


#pylint: disable=R0921
class SignedData(CMSBase):
    """
    Represents signed message (signeddata CMS type)
    """
    @staticmethod
    def create(data, cert, pkey, flags=Flags.BINARY, certs=None):
        """
            Creates SignedData message by signing data with pkey and
            certificate.

            @param data - data to sign
            @param cert - signer's certificate
            @param pkey - pkey object with private key to sign
            @param flags - OReed combination of Flags constants
            @param certs - list of X509 objects to include into CMS
        """
        if not pkey.cansign:
            raise ValueError("Specified keypair has no private part")
        if cert.pubkey != pkey:
            raise ValueError("Certificate doesn't match public key")
        bio = Membio(data)
        if certs is not None and len(certs) > 0:
            certstack = StackOfX509(certs)
        else:
            certstack = None
        ptr = libcrypto.CMS_sign(cert.cert, pkey.ptr, certstack, bio.bio, flags)
        if ptr is None:
            raise CMSError("signing message")
        return SignedData(ptr)
    def sign(self, cert, pkey, digest_type=None, data=None, flags=Flags.BINARY):
        """
            Adds another signer to already signed message
            @param cert - signer's certificate
            @param pkey - signer's private key
            @param digest_type - message digest to use as DigestType object
                (if None - default for key would be used)
            @param data - data to sign (if detached and
                    Flags.REUSE_DIGEST is not specified)
            @param flags - ORed combination of Flags consants
        """
        if not pkey.cansign:
            raise ValueError("Specified keypair has no private part")
        if cert.pubkey != pkey:
            raise ValueError("Certificate doesn't match public key")
        if libcrypto.CMS_add1_signer(self.ptr, cert.cert, pkey.ptr,
                                          digest_type.digest, flags) is None:
            raise CMSError("adding signer")
        if flags & Flags.REUSE_DIGEST == 0:
            if data is not None:
                bio = Membio(data)
                biodata = bio.bio
            else:
                biodata = None
            res = libcrypto.CMS_final(self.ptr, biodata, None, flags)
            if res <= 0:
                raise CMSError("Cannot finalize CMS")
    def verify(self, store, flags, data=None, certs=None):
        """
        Verifies signature under CMS message using trusted cert store

        @param store -  X509Store object with trusted certs
        @param flags - OR-ed combination of flag consants
        @param data - message data, if messge has detached signature
        param certs - list of certificates to use during verification
                If Flags.NOINTERN is specified, these are only
                sertificates to search for signing certificates
        @returns True if signature valid, False otherwise
        """
        bio = None
        if data != None:
            bio_obj = Membio(data)
            bio = bio_obj.bio
        if certs is not None and len(certs) > 0:
            certstack = StackOfX509(certs)
        else:
            certstack = None
        res = libcrypto.CMS_verify(self.ptr, certstack, store.store, bio,
                                   None, flags)
        return res > 0

    @property
    def signers(self):
        """
        Return list of signer's certificates
        """
        signerlist = libcrypto.CMS_get0_signers(self.ptr)
        if signerlist is None:
            raise CMSError("Cannot get signers")
        return StackOfX509(ptr=signerlist, disposable=False)

    @property
    def data(self):
        """
        Returns signed data if present in the message
        """
        bio = Membio()
        if not libcrypto.CMS_verify(self.ptr, None, None, None, bio.bio,
                                    Flags.NO_VERIFY):
            raise CMSError("extract data")
        return str(bio)

    def addcert(self, cert):
        """
        Adds a certificate (probably intermediate CA) to the SignedData
        structure
        """
        if libcrypto.CMS_add1_cert(self.ptr, cert.cert) <= 0:
            raise CMSError("Cannot add cert")
    def addcrl(self, crl):
        """
        Adds a CRL to the signed data structure
        """
        raise NotImplementedError
    @property
    def certs(self):
        """
        List of the certificates contained in the structure
        """
        certstack = libcrypto.CMS_get1_certs(self.ptr)
        if certstack is None:
            raise CMSError("getting certs")
        return StackOfX509(ptr=certstack, disposable=True)
    @property
    def crls(self):
        """
        List of the CRLs contained in the structure
        """
        raise NotImplementedError

class EnvelopedData(CMSBase):
    """
    Represents EnvelopedData CMS, i.e. message encrypted with session
    keys, encrypted with recipient's public keys
    """
    @staticmethod
    def create(recipients, data, cipher, flags=0):
        """
        Creates and encrypts message
        @param recipients - list of X509 objects
        @param data - contents of the message
        @param cipher - CipherType object
        @param flags - flag
        """
        recp = StackOfX509(recipients)
        bio = Membio(data)
        cms_ptr = libcrypto.CMS_encrypt(recp.ptr, bio.bio, cipher.cipher_type,
                                        flags)
        if cms_ptr is None:
            raise CMSError("encrypt EnvelopedData")
        return EnvelopedData(cms_ptr)

    def decrypt(self, pkey, cert, flags=0):
        """
        Decrypts message
        @param pkey - private key to decrypt
        @param cert - certificate of this private key (to find
            neccessary RecipientInfo
        @param flags - flags
        @returns - decrypted data
        """
        if not pkey.cansign:
            raise ValueError("Specified keypair has no private part")
        if pkey != cert.pubkey:
            raise ValueError("Certificate doesn't match private key")
        bio = Membio()
        res = libcrypto.CMS_decrypt(self.ptr, pkey.ptr, cert.ccert, None,
                                    bio.bio, flags)
        if res <= 0:
            raise CMSError("decrypting CMS")
        return str(bio)

class EncryptedData(CMSBase):
    """
    Represents encrypted data CMS structure, i.e. encrypted
    with symmetric key, shared by sender and recepient.
    """
    @staticmethod
    def create(data, cipher, key, flags=0):
        """
        Creates an EncryptedData message.
        @param data data to encrypt
        @param cipher cipher.CipherType object represening required
                cipher type
        @param key - byte array used as simmetic key
        @param flags - OR-ed combination of Flags constant
        """
        bio = Membio(data)
        ptr = libcrypto.CMS_EncryptedData_encrypt(bio.bio, cipher.cipher_type,
                                                  key, len(key), flags)
        if ptr is None:
            raise CMSError("encrypt data")
        return EncryptedData(ptr)

    def decrypt(self, key, flags=0):
        """
        Decrypts encrypted data message
        @param key - symmetic key to decrypt
        @param flags - OR-ed combination of Flags constant
        """
        bio = Membio()
        if libcrypto.CMS_EncryptedData_decrypt(self.ptr, key, len(key), None,
                                               bio.bio, flags) <= 0:
            raise CMSError("decrypt data")
        return str(bio)

__all__ = ['CMS', 'CMSError', 'Flags', 'SignedData', 'EnvelopedData',
           'EncryptedData']

libcrypto.CMS_add1_cert.restype = c_int
libcrypto.CMS_add1_cert.argtypes = (c_void_p, c_void_p)
libcrypto.CMS_decrypt.restype = c_int
libcrypto.CMS_decrypt.argtypes = (c_void_p, c_void_p, c_void_p,
                                  c_void_p, c_void_p, c_uint)
libcrypto.CMS_encrypt.restype = c_void_p
libcrypto.CMS_encrypt.argtypes = (c_void_p, c_void_p, c_void_p, c_uint)
libcrypto.CMS_EncryptedData_decrypt.restype = c_int
libcrypto.CMS_EncryptedData_decrypt.argtypes = (c_void_p, c_char_p, c_size_t,
                                                c_void_p, c_void_p, c_uint)
libcrypto.CMS_EncryptedData_encrypt.restype = c_void_p
libcrypto.CMS_EncryptedData_encrypt.argtypes = (c_void_p, c_void_p, c_char_p,
                                                c_size_t, c_uint)
libcrypto.CMS_final.restype = c_int
libcrypto.CMS_final.argtypes = (c_void_p, c_void_p, c_void_p, c_uint)
libcrypto.CMS_get0_signers.restype = c_void_p
libcrypto.CMS_get0_signers.argtypes = (c_void_p, )
libcrypto.CMS_get1_certs.restype = c_void_p
libcrypto.CMS_get1_certs.argtypes = (c_void_p, )
libcrypto.CMS_sign.restype = c_void_p
libcrypto.CMS_sign.argtypes = (c_void_p, c_void_p, c_void_p, c_void_p, c_uint)
libcrypto.CMS_add1_signer.restype = c_void_p
libcrypto.CMS_add1_signer.argtypes = (c_void_p, c_void_p, c_void_p,
                                           c_void_p, c_uint)
libcrypto.CMS_verify.restype = c_int
libcrypto.CMS_verify.argtypes = (c_void_p, c_void_p, c_void_p, c_void_p,
                                 c_void_p, c_int)
libcrypto.d2i_CMS_bio.restype = c_void_p
libcrypto.d2i_CMS_bio.argtypes = (c_void_p, POINTER(c_void_p))
libcrypto.i2d_CMS_bio.restype = c_int
libcrypto.i2d_CMS_bio.argtypes = (c_void_p, c_void_p)
libcrypto.PEM_read_bio_CMS.restype = c_void_p
libcrypto.PEM_read_bio_CMS.argtypes = (c_void_p, POINTER(c_void_p),
                                       c_void_p, c_void_p)
libcrypto.PEM_write_bio_CMS.restype = c_int
libcrypto.PEM_write_bio_CMS.argtypes = (c_void_p, c_void_p)
